@@ -9,7 +9,7 @@
 //  Module:         Multithreaded AFF (AAFF = Advanced AFF)
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -64,6 +64,7 @@
 #define AFF_SEGNAME_PAGESIZE          "pagesize"
 #define AFF_SEGNAME_BADSECTORS        "badsectors"
 #define AFF_SEGNAME_MD5               "md5"
+#define AFF_SEGNAME_SHA1              "sha1"
 #define AFF_SEGNAME_SHA256            "sha256"
 #define AFF_SEGNAME_DURATION          "acquisition_seconds"
 #define AFF_SEGNAME_PAGE              "page"
@@ -113,6 +114,7 @@ typedef struct _t_Aaff
 } t_Aaff;
 
 #define AAFF_MD5_LEN                16
+#define AAFF_SHA1_LEN               20
 #define AAFF_SHA256_LEN             32
 #define AAFF_BADSECTORMARKER_MAXLEN 65536
 unsigned char AaffBadSectorMarker[AAFF_BADSECTORMARKER_MAXLEN];
@@ -285,12 +287,13 @@ APIRET AaffOpen (t_pAaff *ppAaff, const char *pFilename, unsigned long long Devi
 }
 //lint -restore
 
-APIRET AaffClose (t_pAaff *ppAaff, unsigned long long BadSectors, const unsigned char *pMD5, const unsigned char *pSHA256, uint Duration)
+APIRET AaffClose (t_pAaff *ppAaff, unsigned long long BadSectors, const unsigned char *pMD5, const unsigned char *pSHA1, const unsigned char *pSHA256, uint Duration)
 {
    t_pAaff pAaff = *ppAaff;
 
    CHK (AaffWriteSegmentU64 (pAaff, AFF_SEGNAME_BADSECTORS, BadSectors))
    CHK (AaffWriteSegment    (pAaff, AFF_SEGNAME_MD5       , 0, pMD5   , AAFF_MD5_LEN   ))
+   CHK (AaffWriteSegment    (pAaff, AFF_SEGNAME_SHA1      , 0, pSHA1  , AAFF_SHA1_LEN  ))
    CHK (AaffWriteSegment    (pAaff, AFF_SEGNAME_SHA256    , 0, pSHA256, AAFF_SHA256_LEN))
    CHK (AaffWriteSegmentArg (pAaff, AFF_SEGNAME_DURATION  , Duration))
 
@@ -309,31 +312,31 @@ APIRET AaffClose (t_pAaff *ppAaff, unsigned long long BadSectors, const unsigned
 }
 
 //lint -save -esym(613,pPreProcess)   Possible use of null pointer
-APIRET AaffPreprocess (t_pAaffPreprocess *ppPreprocess, unsigned char *pDataIn, unsigned int DataLenIn, unsigned char *pDataOut, uint DataLenOut)
+APIRET AaffPreprocess (t_pAaffPreprocess pPreprocess, unsigned char *pDataIn, unsigned int DataLenIn, unsigned char *pDataOut, uint DataLenOut)
 {
-   t_pAaffPreprocess pPreProcess;
+//   t_pAaffPreprocess pPreProcess;
    int                rc;
    uLongf             LenOut;
 
-   *ppPreprocess = NULL;
-   pPreProcess = (t_pAaffPreprocess) UTIL_MEM_ALLOC(sizeof(t_AaffPreprocess));
-   if (pPreProcess == NULL)
-      CHK (ERROR_AAFF_MEMALLOC_FAILED)
-   *ppPreprocess = pPreProcess;
-   pPreProcess->Zero       = false;
-   pPreProcess->Compressed = false;
+//   *ppPreprocess = NULL;
+//   pPreProcess = (t_pAaffPreprocess) UTIL_MEM_ALLOC(sizeof(t_AaffPreprocess));
+//   if (pPreProcess == NULL)
+//      CHK (ERROR_AAFF_MEMALLOC_FAILED)
+//   *ppPreprocess = pPreProcess;
+   pPreprocess->Zero       = false;
+   pPreprocess->Compressed = false;
 
    // Check if zero
    // -------------
-   pPreProcess->Zero = UtilIsZero (pDataIn, DataLenIn);
-   if (pPreProcess->Zero)
+   pPreprocess->Zero = UtilIsZero (pDataIn, DataLenIn);
+   if (pPreprocess->Zero)
       return NO_ERROR;
 
    // Try compression
    // ---------------
    LenOut = DataLenOut;
    rc = compress2 ((Bytef *)pDataOut, &LenOut, (Bytef *)pDataIn, DataLenIn, CONFIG (AffCompression));
-   pPreProcess->DataLenOut = LenOut;
+   pPreprocess->DataLenOut = LenOut;
    if (rc != Z_OK)
    {
       if (rc != Z_BUF_ERROR)    // Do not log this one (the destination buffer was too small for the compressed result)
@@ -341,7 +344,7 @@ APIRET AaffPreprocess (t_pAaffPreprocess *ppPreprocess, unsigned char *pDataIn,
       return NO_ERROR;
    }
 
-   pPreProcess->Compressed = (LenOut < DataLenIn);
+   pPreprocess->Compressed = (LenOut < DataLenIn);
 
    return NO_ERROR;
 }
@@ -362,7 +365,7 @@ APIRET AaffWrite (t_pAaff pAaff, t_pAaffPreprocess pPreprocess, const unsigned c
         CHK (AaffWriteSegment (pAaff, &SegmentName[0], AFF_PAGEFLAGS_COMPRESSED_ZLIB, pData, pPreprocess->DataLenOut))
    else CHK (AaffWriteSegment (pAaff, &SegmentName[0], AFF_PAGEFLAGS_UNCOMPRESSED   , pData, DataLen))
 
-   UTIL_MEM_FREE (pPreprocess);
+//   UTIL_MEM_FREE (pPreprocess);
 
    return NO_ERROR;
 }
