@@ -9,7 +9,7 @@
 //  Module:         Multithreaded AEWF (AEWF = Advanced EWF)
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -50,7 +50,7 @@
 // ------------------
 
 // Qt inconsistently defines its little/big endian functions, see QTBUG-21208 on bugreports.qt.nokia.com
-// The following lines implement conversin for uchar and avoid the problems. 
+// The following lines implement conversin for uchar and avoid the problems.
 
 #if (QT_VERSION < 0x040800)
    template <typename T> T qbswap(T source);
@@ -185,8 +185,8 @@ class t_Aewf
       QString          OSVersion;
 
       uint             SegmentFiles;
-      uchar            EwfCompressionLevel;
-      int              ZipCompressionLevel;
+//      uchar            EwfCompressionLevel;   // The compression levels have been made global in order not to depend upon
+//      int              ZipCompressionLevel;   // Aewf handle in preprocessing (this simplifies duplicate image writing)
       QStringList      SegmentFilenameList;
       FILE           *pCurrentSegmentFile;
       quint64          CurrentSectionSectorSeekPos;
@@ -198,7 +198,7 @@ class t_Aewf
       uint             CurrentChunkOffset;           // For knowing the current chunk during verification
 
       t_HashContextMD5 CurrentSegmentHashContextMD5; // For segment file MD5 calculation during verification
-      uchar          *pVerifyBuffer;                 // For uncompression and othe verification tasks
+      uchar          *pVerifyBuffer;                 // For uncompression and other verification tasks
       uint             VerifyBufferLen;
 
       t_Aewf ()
@@ -212,13 +212,13 @@ class t_Aewf
          FileHeaderCache.StartOfFields = qToLittleEndian (AEWF_STARTOFFIELDS);
          FileHeaderCache.SegmentNumber = qToLittleEndian (0);
          FileHeaderCache.EndOfFields   = qToLittleEndian (AEWF_ENDOFFIELDS);
-         switch (CONFIG(EwfCompression))
-         {
-            case LIBEWF_COMPRESSION_NONE: ZipCompressionLevel = 0; EwfCompressionLevel = 0x00; break;
-            case LIBEWF_COMPRESSION_FAST: ZipCompressionLevel = 1; EwfCompressionLevel = 0x01; break;
-            case LIBEWF_COMPRESSION_BEST: ZipCompressionLevel = 9; EwfCompressionLevel = 0x02; break;
-            default: CHK_EXIT (ERROR_AEWF_INVALID_COMPRESSION_LEVEL)
-         }
+//         switch (CONFIG(EwfCompression))
+//         {
+//            case LIBEWF_COMPRESSION_NONE: ZipCompressionLevel = 0; EwfCompressionLevel = 0x00; break;
+//            case LIBEWF_COMPRESSION_FAST: ZipCompressionLevel = 1; EwfCompressionLevel = 0x01; break;
+//            case LIBEWF_COMPRESSION_BEST: ZipCompressionLevel = 9; EwfCompressionLevel = 0x02; break;
+//            default: CHK_EXIT (ERROR_AEWF_INVALID_COMPRESSION_LEVEL)
+//         }
       }
 };
 
@@ -229,6 +229,9 @@ unsigned char *pAewfZeroBlockCompressed;
 ulong           AewfZeroBlockCompressedLen;
 ulong           AewfZeroBlockUnCompressedLen;
 
+int             AewfZipCompressionLevel;
+uchar           AewfEwfCompressionLevel;
+
 // ----------------
 //  Error handling
 // ----------------
@@ -237,7 +240,6 @@ ulong           AewfZeroBlockUnCompressedLen;
    if ((Fn) != 1)                         \
       CHK (ERROR_AEWF_CANNOT_WRITE_FILE)
 
-
 // -------------------
 //  Utility functions
 // -------------------
@@ -256,12 +258,15 @@ static int AewfCRC (const void *pBuffer, size_t Len)
 // Nobody exactly seems to know what follows then. Joachim Metz writes that maybe [AA would follow. If so, it
 // doesn't help a lot, as it only doubles the available naming space (the following ASCII character is \ and
 // thus cannot be used on Windows systems with their inconsistent usage of \ and / ).
-// So, I decided to continue with ZZZxxx, where xxx represents a numbers from 000 to ZZZ in base36 notation (i.e.
-// 0-9 and A-Z). After that, it would continue with ZZZxxxx and so on.
-// Old-fashionned Windows-file systems only allow 3 letters for the extension. If ever this occurs the acquisition
-// will fail. So, we simply tell the user 'bad luck' and that's all? Yes, but, wait a moment:
-//   a) nobody said he should use old-fashionned Windows FS
-//   b) why use Windows at all?
+// Guymager suports 2 ways for continuing beyond extension ZZZ:
+// OLD method:
+//    Here I decided to continue with ZZZxxx, where xxx represents a numbers from 000 to ZZZ in base36 notation (i.e.
+//    0-9 and A-Z). After that, it would continue with ZZZxxxx and so on.
+// FTK method
+//   According to tests made by Willi Spiegel, FTK continues with E14972, E14973 and so on.
+//
+// Reminder: Old-fashionned Windows-file systems only allow 3 letters for the extension. So, extensions beyond ZZZ
+// must be avoided there.
 
 const QString AewfGetExtension (uint SegmentFileNr)
 {
@@ -285,8 +290,20 @@ const QString AewfGetExtension (uint SegmentFileNr)
       }
       else
       {
-         SegmentFileNr -= 25*26*26 + 25*26 + 25; // substract ZZZ
-         Extension = QString("ZZZ%1").arg(SegmentFileNr,3,36,QChar('0')).toUpper();
+         switch (CONFIG(EwfNaming))
+         {
+            case EWFNAMING_OLD:
+               SegmentFileNr -= 26*26*26; // substract ZZZ+1 -> so, for the segment following ZZZ the results is 0
+               Extension = QString("ZZZ%1").arg(SegmentFileNr,3,36,QChar('0')).toUpper();
+               break;
+            case EWFNAMING_FTK:
+               SegmentFileNr -= 26*26*26; // substract ZZZ+1 -> so, for the segment following ZZZ the results is 0
+               SegmentFileNr += 14972;
+               Extension = QString("E%1").arg(SegmentFileNr);
+               break;
+            default:
+               CHK_EXIT (ERROR_AEWF_INVALID_EWFNAMING)
+         }
       }
    }
 
@@ -295,7 +312,8 @@ const QString AewfGetExtension (uint SegmentFileNr)
 
 // AewfCompress: No need to have an extra check if compression is set to "none". If this is the case, pAewf->ZipCompressionLevel
 // is set to 0 and compress2 won't do any compression (see libz / compress2 documentation).
-static APIRET AewfCompress (const t_pAewf pAewf, void *pDst, ulong *pDstLen, const void* pSrc, unsigned SrcLen)
+// As well, no need to check for compression level "empty", as AewfZipCompressionLevel is set to 0 in this case (see AewfInit)
+static APIRET AewfCompress (void *pDst, ulong *pDstLen, const void* pSrc, unsigned SrcLen)
 {
    int rc;
 
@@ -308,7 +326,7 @@ static APIRET AewfCompress (const t_pAewf pAewf, void *pDst, ulong *pDstLen, con
    }
    else
    {
-      rc = compress2 ((Bytef*)pDst, pDstLen, (Bytef*)pSrc, SrcLen, pAewf->ZipCompressionLevel);
+      rc = compress2 ((Bytef*)pDst, pDstLen, (Bytef*)pSrc, SrcLen, AewfZipCompressionLevel);
       if (rc != Z_OK)
          return ERROR_AEWF_COMPRESSION_FAILED;
    }
@@ -318,12 +336,20 @@ static APIRET AewfCompress (const t_pAewf pAewf, void *pDst, ulong *pDstLen, con
 static APIRET AewfUncompress (void *pUncompressed, ulong *pUncompressedLen, void *pCompressed, ulong CompressedLen)
 {
    int rc;
-   
+
+//   #define AEWF_SIMULATE_IMAGE_FAILURE
+   #ifdef AEWF_SIMULATE_IMAGE_FAILURE
+      #warning "AEWF_SIMULATE_IMAGE_FAILURE debugging code active"
+      static int Debug=0;
+      if (Debug++ == 5654)
+         return ERROR_AEWF_UNCOMPRESS_FAILED;
+   #endif
+
    if ((CompressedLen == AewfZeroBlockCompressedLen) &&
        (memcmp(pCompressed, pAewfZeroBlockCompressed, CompressedLen) == 0))
    {
       if (*pUncompressedLen < AewfZeroBlockUnCompressedLen)
-         return ERROR_AEWF_UNCOMPRESS_FAILED;
+         return ERROR_AEWF_UNCOMPRESS_ZEROBLOCK_FAILED;
       *pUncompressedLen = AewfZeroBlockUnCompressedLen;
       memset (pUncompressed, 0, AewfZeroBlockUnCompressedLen);
    }
@@ -516,7 +542,7 @@ static APIRET AewfWriteSectionHeader (t_pAewf pAewf)  // Writes the sections hea
 
    CompressedLen = UtilGetMaxZcompressedBufferSize (ArrLen);
    pCompressed   = UTIL_MEM_ALLOC (CompressedLen);
-   CHK (AewfCompress (pAewf, pCompressed, &CompressedLen, pArr, ArrLen))
+   CHK (AewfCompress (pCompressed, &CompressedLen, pArr, ArrLen))
 
    rc = AewfWriteSection (pAewf, AEWF_SECTIONNAME_HEADER2, pCompressed, CompressedLen);
    rc = AewfWriteSection (pAewf, AEWF_SECTIONNAME_HEADER2, pCompressed, CompressedLen);
@@ -537,7 +563,7 @@ static APIRET AewfWriteSectionHeader (t_pAewf pAewf)  // Writes the sections hea
    ByteArr = HeaderData.toLatin1();
    CompressedLen = UtilGetMaxZcompressedBufferSize (ByteArr.length());
    pCompressed   = UTIL_MEM_ALLOC (CompressedLen);
-   CHK (AewfCompress (pAewf, pCompressed, &CompressedLen, ByteArr.constData(), ByteArr.length()))
+   CHK (AewfCompress (pCompressed, &CompressedLen, ByteArr.constData(), ByteArr.length()))
 
    rc = AewfWriteSection (pAewf, AEWF_SECTIONNAME_HEADER, pCompressed, CompressedLen);
    UTIL_MEM_FREE (pCompressed);
@@ -562,7 +588,7 @@ static APIRET AewfWriteSectionVolume (t_Aewf *pAewf, bool NameVolume=true)
    VolumeContents.BytesPerSector   = qToLittleEndian ((uint)    pAewf->SectorSize);
    VolumeContents.SectorCount      = qToLittleEndian ((quint64) pAewf->DeviceSize / pAewf->SectorSize);
    VolumeContents.MediaFlags       = qToLittleEndian ((uchar)  (pAewf->MediaFlags | AEWF_MEDIAFLAGS_IMAGE));
-   VolumeContents.CompressionLevel = qToLittleEndian ((uchar)   pAewf->EwfCompressionLevel);
+   VolumeContents.CompressionLevel = qToLittleEndian ((uchar)   AewfEwfCompressionLevel);
    VolumeContents.ErrorBlockSize   = qToLittleEndian ((uint)    1);
    VolumeContents.Checksum         = qToLittleEndian (AewfCRC (&VolumeContents, offsetof(t_AewfSectionVolumeContents, Checksum)));
    if (NameVolume)
@@ -600,13 +626,17 @@ static APIRET AewfWriteSectionTable (t_Aewf *pAewf)
    pTableContents->BaseOffset = qToLittleEndian (pAewf->ChunkBaseOffset);
    pTableContents->CRC1       = qToLittleEndian (AewfCRC(pTableContents, offsetof(t_AewfSectionTableContents, CRC1)));
    memcpy (&pTableContents->OffsetArr[0], &pAewf->ChunkOffsetArr[0], OffsetArrSize);     // pAewf->ChunkOffsetArr already is in little endian!
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Warray-bounds"
    pCRC2 = (unsigned int *) (&pTableContents->OffsetArr[0] + pAewf->ChunkOffsets);
+#pragma GCC diagnostic pop
    *pCRC2 = qToLittleEndian (AewfCRC(pTableContents->OffsetArr, OffsetArrSize));
 
    CHK (AewfWriteSection (pAewf, AEWF_SECTIONNAME_TABLE , pTableContents, sizeof(t_AewfSectionTableContents)+OffsetArrSize))
    CHK (AewfWriteSection (pAewf, AEWF_SECTIONNAME_TABLE2, pTableContents, sizeof(t_AewfSectionTableContents)+OffsetArrSize))
    UTIL_MEM_FREE (pTableContents);
 
+   memset (pAewf->ChunkOffsetArr, 0, sizeof(pAewf->ChunkOffsetArr));
    pAewf->ChunkOffsets    = 0;
    pAewf->ChunkBaseOffset = 0;
 
@@ -721,38 +751,72 @@ APIRET AewfOpen (t_pAewf *ppAewf, const char *pFilename,
 }
 
 
-APIRET AewfPreprocess (t_Aewf *pAewf, t_pAewfPreprocess *ppPreprocess, uchar *pDataIn, unsigned int DataLenIn, uchar *pDataOut, unsigned int DataLenOut)
+APIRET AewfPreprocess (t_pAewfPreprocess pPreprocess, uchar *pDataIn, unsigned int DataLenIn, uchar *pDataOut, unsigned int DataLenOut)
 {
-   t_pAewfPreprocess pPreProcess;
+//   t_pAewfPreprocess pPreProcess;
    int                rc;
    uLongf             LenOut;
    uint             *pCRC;
-
-   *ppPreprocess = NULL;
-   pPreProcess = (t_pAewfPreprocess) UTIL_MEM_ALLOC(sizeof(t_AewfPreprocess));
-   if (pPreProcess == NULL)
-      CHK (ERROR_AEWF_MEMALLOC_FAILED)
-   *ppPreprocess = pPreProcess;
-   pPreProcess->Compressed = false;
+   uint               MaxLenOut;
+#ifdef AEWF_COMPRESSION_DEBUG
+   static bool        FirstCall=true;
+   static int         CharCount=0;
+#endif
+//   *ppPreprocess = NULL;
+//   pPreProcess = (t_pAewfPreprocess) UTIL_MEM_ALLOC(sizeof(t_AewfPreprocess));
+//   if (pPreProcess == NULL)
+//      CHK (ERROR_AEWF_MEMALLOC_FAILED)
+//   *ppPreprocess = pPreProcess;
+   pPreprocess->Compressed = false;
 
    if (CONFIG(EwfCompression) != LIBEWF_COMPRESSION_NONE)
    {
       LenOut = DataLenOut;
-      rc = AewfCompress (pAewf, pDataOut, &LenOut, pDataIn, DataLenIn);
+      rc = AewfCompress (pDataOut, &LenOut, pDataIn, DataLenIn);
 
-      pPreProcess->DataLenOut = LenOut;
+      pPreprocess->DataLenOut = LenOut;
+      MaxLenOut = (unsigned int) (DataLenIn * CONFIG(EwfCompressionThreshold));
+      MaxLenOut = (unsigned int) GETMIN (DataLenIn-1, MaxLenOut); // Avoid potential float operation side effects
+#ifdef AEWF_COMPRESSION_DEBUG
+      if (FirstCall)
+      {
+         printf ("\nAEWF -- Chunk size:                      %u"                                      , DataLenIn);
+         printf ("\nAEWF -- Min. required compression ratio: %1.3f (EwfCompressionThreshold)"         , CONFIG(EwfCompressionThreshold));
+         printf ("\nAEWF -- Max. compressed size:            %u (EwfCompressionThreshold * ChunkSize)", MaxLenOut);
+         printf ("\nAEWF -- List of chunks (. = not compressed because compressed data is larger than uncompressed data,\n"
+                   "                        M = not compressed because compressed data is larger max. compressed size,\n"
+                   "                        c = Compressed\n");
+         FirstCall = false;
+      }
+#endif
       if (rc != NO_ERROR)
            LOG_ERROR ("compress2 returned %d - treating this block as uncompressed in order to resume", rc)
-//      else pPreProcess->Compressed = (LenOut < (DataLenIn + sizeof(unsigned int))); // This leads to problems in Encase
-      else pPreProcess->Compressed = (LenOut < DataLenIn);
+      else pPreprocess->Compressed = (LenOut <= MaxLenOut);
    }
 
-   if (!pPreProcess->Compressed)  // Add the CRC to the end of the non-compressed data. The FIFO buffer is
+   if (!pPreprocess->Compressed)  // Add the CRC to the end of the non-compressed data. The FIFO buffer is
    {                              // big enough for this (see extra space calculation)
       pCRC = (unsigned int *) &pDataIn[DataLenIn];
       *pCRC = qToLittleEndian (AewfCRC (pDataIn, DataLenIn));
-      pPreProcess->DataLenOut = DataLenIn + sizeof (unsigned int);
+      pPreprocess->DataLenOut = DataLenIn + sizeof (unsigned int);
+#ifdef AEWF_COMPRESSION_DEBUG
+      if (LenOut < DataLenIn)
+           printf("M");
+      else printf(".");
+#endif
    }
+#ifdef AEWF_COMPRESSION_DEBUG
+   else
+   {
+      printf("c");
+   }
+
+   if (++CharCount == 80)
+   {
+      printf ("\n");
+      CharCount=0;
+   }
+#endif
    return NO_ERROR;
 }
 
@@ -761,11 +825,12 @@ APIRET AewfWrite (t_Aewf *pAewf, t_AewfPreprocess *pPreprocess, const uchar *pDa
    quint64 SectionTableSize;
    quint64 MinRemainingSize;
    uint    OffsetArrEntry;
+   uint    NewLen;
 
    if (pAewf->pCurrentSegmentFile == NULL)
       CHK (ERROR_AEWF_NO_FILE_OPEN)
 
-   //printf ("\nChunk: %d  %u bytes - %s", pAewf->ChunkOffsets, DataLen, pPreprocess->Compressed ? "compressed" : "uc");
+//   printf ("\nChunk: %d  %u bytes - %s", pAewf->ChunkOffsets, DataLen, pPreprocess->Compressed ? "compressed" : "uc");
 
    // Check if enough space remaining in current segment file
    // -------------------------------------------------------
@@ -791,9 +856,14 @@ APIRET AewfWrite (t_Aewf *pAewf, t_AewfPreprocess *pPreprocess, const uchar *pDa
 
    // Check if current sector section or offset table is full
    // -------------------------------------------------------
-   if (((pAewf->CurrentSectionSectorContentSize + DataLen) > AEWF_MAX_SIZE_SECTIONSECTOR) || // Unlike libewf, we prefer to have the whole section "sectors" <= 2GiB (libewf
-        (pAewf->ChunkOffsets == AEWF_MAX_CHUNK_OFFSETS))                                     // allows to have the last chunk start below the 2GiB limit and span beyond)
+   NewLen = pAewf->CurrentSectionSectorContentSize + DataLen;
+   if (pAewf->ChunkOffsets)                                         // We have to add the offset of the first chunk, which normally is 76 (see also comment related to
+      NewLen += pAewf->ChunkOffsetArr[0] & ~AEWF_COMPRESSED_CHUNK;  // ewfacquire behaviour below). However, we only know this for sure after a first chunk has been written.
+                                                                    // This is no problem, as the first chunk will never be so big that it would touch the 2GB limit.
+   if ((NewLen > AEWF_MAX_SIZE_SECTIONSECTOR) ||                    // Unlike libewf, we prefer to have the whole section "sectors" <= 2GiB (libewf allows to have the
+       (pAewf->ChunkOffsets == AEWF_MAX_CHUNK_OFFSETS))             // last chunk start below the 2GiB limit and span beyond).
    {
+//      printf ("\nNew table -- %llu %d %u %u", pAewf->CurrentSectionSectorContentSize, DataLen, NewLen, pAewf->ChunkOffsets);
       CHK (AewfFinishSectionSectors (pAewf))
       CHK (AewfWriteSectionTable    (pAewf))
    }
@@ -815,12 +885,12 @@ APIRET AewfWrite (t_Aewf *pAewf, t_AewfPreprocess *pPreprocess, const uchar *pDa
    CHK (AewfWriteFile (pAewf, pData, DataLen))
    pAewf->CurrentSectionSectorContentSize += DataLen;
 
-   UTIL_MEM_FREE (pPreprocess);
+//   UTIL_MEM_FREE (pPreprocess);
 
    return NO_ERROR;
 }
 
-APIRET AewfClose (t_pAewf *ppAewf, QList<quint64> &BadSectors, const uchar *pMD5, const uchar */*pSHA256*/)
+APIRET AewfClose (t_pAewf *ppAewf, QList<quint64> &BadSectors, const uchar *pMD5, const uchar */*pSHA1*/, const uchar */*pSHA256*/)
 {
    t_Aewf *pAewf = *ppAewf;
 
@@ -1020,7 +1090,7 @@ static APIRET AewfReadTable (t_Aewf *pAewf)
 
 // AewfReadNextChunk: Besides reading data chunk, the function returns image filenames and image file hashes "from time to time", i.e. when
 // finishing with the data found in a segment file. If ever this is the case, all 3 params (pSegmentFilename, pMD5Digest and pMD5Valid) are correctly
-// filled in. If not, ImageFilename simply is empty.
+// filled in. If not, SegmentFilename simply is empty.
 // AewfReadNextChunk must be called again and again until pFinished is set to true, even if no data is returned. This must be done in order to have
 // the function return all segment file names with correct MD5 values.
 
@@ -1127,7 +1197,12 @@ APIRET AewfReadNextChunk (t_Aewf *pAewf, uchar *pData, uint *pDataLen, QString *
    if (!Compressed)
    {
       if (ReadLen > *pDataLen)
+      {
+         LOG_INFO ("ChunkBaseOffset=%llu CurrentChunkOffset=%u", pAewf->ChunkBaseOffset, pAewf->CurrentChunkOffset)
+         LOG_INFO ("ReadLen=%u DataLen=%u"                     , ReadLen, *pDataLen)
+         LOG_INFO ("Offset=%u NextOffset=%u"                   , Offset, NextOffset)
          CHK_LOG (pAewf, ERROR_AEWF_VERIFY_BUFFER_TOO_SMALL)
+      }
       CHK (AewfReadFile (pAewf, pData, ReadLen))
 
       *pDataLen = ReadLen - sizeof (unsigned int);  // CRC is at the end of uncompressed data
@@ -1142,17 +1217,11 @@ APIRET AewfReadNextChunk (t_Aewf *pAewf, uchar *pData, uint *pDataLen, QString *
       CHK (AewfReadFile            (pAewf, pAewf->pVerifyBuffer, ReadLen))
 
       DataLen = *pDataLen;
-//      rc = uncompress (pData, &DataLen, pAewf->pVerifyBuffer, ReadLen);
       rc = AewfUncompress (pData, &DataLen, pAewf->pVerifyBuffer, ReadLen);
-      
+
       *pDataLen = DataLen;
-//      if (rc != Z_OK)
       if (rc != NO_ERROR)
-      {
-//         LOG_ERROR ("Zlib uncompress returned %d", rc)
-//         CHK_LOG (pAewf, ERROR_AEWF_UNCOMPRESS_FAILED)
          CHK_LOG (pAewf, rc)
-      }
    }
 
    return NO_ERROR;
@@ -1165,7 +1234,7 @@ APIRET AewfReadNextChunk (t_Aewf *pAewf, uchar *pData, uint *pDataLen, QString *
 unsigned int AewfPreprocessExtraSpace (uint FifoBlockSize)
 {
    // AEWF needs some extra FIFO block buffer space in order to do the compression or to add the CRC
-   // at the ende of the uncompressed data. We take the max. value of both in order to be sure that
+   // at the end of the uncompressed data. We take the max. value of both in order to be sure that
    // enough space is available for both.
 
    uint ExtraZ               = UtilGetMaxZcompressedBufferSize (FifoBlockSize) - FifoBlockSize;
@@ -1192,6 +1261,7 @@ APIRET AewfInit (void)
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AEWF_CANNOT_READ_DATA           ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AEWF_DATA_CRC_ERROR             ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AEWF_UNCOMPRESS_FAILED          ))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AEWF_UNCOMPRESS_ZEROBLOCK_FAILED))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AEWF_WRONG_SEGMENT_NUMBER       ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AEWF_WRONG_SECTION_CRC          ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AEWF_ENDING_SECTION_MISSING     ))
@@ -1201,41 +1271,53 @@ APIRET AewfInit (void)
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AEWF_TOO_MANY_TABLE_OFFSETS     ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AEWF_DATA_FOLLOWING_LAST_SECTION))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AEWF_WRONG_SECTION_LENGTH       ))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AEWF_INVALID_EWFNAMING          ))
+
+   // Compresison levels
+   // ------------------
+   switch (CONFIG(EwfCompression))
+   {
+      case LIBEWF_COMPRESSION_NONE : AewfZipCompressionLevel = 0; AewfEwfCompressionLevel = 0x00; break;
+      case LIBEWF_COMPRESSION_EMPTY: AewfZipCompressionLevel = 0; AewfEwfCompressionLevel = 0x00; break;
+      case LIBEWF_COMPRESSION_FAST : AewfZipCompressionLevel = 1; AewfEwfCompressionLevel = 0x01; break;
+      case LIBEWF_COMPRESSION_BEST : AewfZipCompressionLevel = 9; AewfEwfCompressionLevel = 0x02; break;
+      default: CHK_EXIT (ERROR_AEWF_INVALID_COMPRESSION_LEVEL)
+   }
 
    // Compressed zero buffer initialisation
    // -------------------------------------
    unsigned char  *pZeroBuff;
    int              rc;
-   
+
    AewfZeroBlockUnCompressedLen = CONFIG (FifoBlockSizeEWF);             // Alloc and initialise zero and compression buffers
    AewfZeroBlockCompressedLen   = UtilGetMaxZcompressedBufferSize (AewfZeroBlockUnCompressedLen);
-   pZeroBuff                    = (unsigned char *) malloc (AewfZeroBlockUnCompressedLen); 
+   pZeroBuff                    = (unsigned char *) malloc (AewfZeroBlockUnCompressedLen);
    pAewfZeroBlockCompressed     = (unsigned char *) malloc (AewfZeroBlockUnCompressedLen);
    if ((pZeroBuff == NULL) || (pAewfZeroBlockCompressed == NULL))
       CHK (ERROR_AEWF_MEMALLOC_FAILED)
    memset (pZeroBuff, 0, AewfZeroBlockUnCompressedLen);
-   
+
    rc = compress2 ((Bytef*)pAewfZeroBlockCompressed, &AewfZeroBlockCompressedLen, (Bytef*)pZeroBuff, AewfZeroBlockUnCompressedLen, 9); // Compress zero buff
    if (rc != Z_OK)
       CHK (ERROR_AEWF_COMPRESSION_FAILED)
-   
+
    free (pAewfZeroBlockCompressed);                                   // Realloc compressed to the size that is really needed
    pAewfZeroBlockCompressed = (unsigned char *) malloc (AewfZeroBlockCompressedLen);
    if (pAewfZeroBlockCompressed == NULL)
       CHK (ERROR_AEWF_MEMALLOC_FAILED)
-   
+
    rc = compress2 ((Bytef*)pAewfZeroBlockCompressed, &AewfZeroBlockCompressedLen, (Bytef*)pZeroBuff, AewfZeroBlockUnCompressedLen, 9);
    if (rc != Z_OK)
       CHK (ERROR_AEWF_COMPRESSION_FAILED)
-      
+
    free (pZeroBuff);
-   
+
    return NO_ERROR;
 }
 
 APIRET AewfDeInit (void)
 {
    free (pAewfZeroBlockCompressed);
-   
+
    return NO_ERROR;
 }
