@@ -9,7 +9,7 @@
 //  Module:         Multithreaded AEWF (AEWF = Advanced EWF)
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -70,9 +70,9 @@ APIRET AewfOpen (t_pAewf *ppAewf, const char *pFilename,
                                   QString Description, QString CaseNumber,  QString EvidenceNumber, QString Examiner,
                                   QString Notes,       QString DeviceModel, QString SerialNumber,   QString ImagerVersion,
                                   QString OSVersion);
-APIRET AewfPreprocess    (t_pAewf   pAewf, t_pAewfPreprocess *ppPreprocess, uchar *pDataIn, uint DataLenIn, uchar *pDataOut, uint DataLenOut);
+APIRET AewfPreprocess    (                 t_pAewfPreprocess pPreprocess, uchar *pDataIn, uint DataLenIn, uchar *pDataOut, uint DataLenOut);
 APIRET AewfWrite         (t_pAewf   pAewf, t_pAewfPreprocess pPreprocess, const uchar *pData, uint DataLen);
-APIRET AewfClose         (t_pAewf *ppAewf, QList<quint64> &BadSectors, const uchar *pMD5, const uchar *pSHA256);
+APIRET AewfClose         (t_pAewf *ppAewf, QList<quint64> &BadSectors, const uchar *pMD5, const uchar *pSHA1, const uchar *pSHA256);
 
 // Read access functions
 // ---------------------
@@ -110,6 +110,7 @@ enum
    ERROR_AEWF_CANNOT_READ_DATA,
    ERROR_AEWF_DATA_CRC_ERROR,
    ERROR_AEWF_UNCOMPRESS_FAILED,
+   ERROR_AEWF_UNCOMPRESS_ZEROBLOCK_FAILED,
    ERROR_AEWF_WRONG_SEGMENT_NUMBER,
    ERROR_AEWF_WRONG_SECTION_CRC,
    ERROR_AEWF_ENDING_SECTION_MISSING,
@@ -118,7 +119,8 @@ enum
    ERROR_AEWF_TABLE_WRONG_CRC2,
    ERROR_AEWF_TOO_MANY_TABLE_OFFSETS,
    ERROR_AEWF_DATA_FOLLOWING_LAST_SECTION,
-   ERROR_AEWF_WRONG_SECTION_LENGTH
+   ERROR_AEWF_WRONG_SECTION_LENGTH,
+   ERROR_AEWF_INVALID_EWFNAMING
 };
 
 #endif
