@@ -7,7 +7,7 @@
 //                  Section Nouvelles Technologies
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -61,6 +61,7 @@ const int CFG_MAX_MENU_NAME_LEN     =  255;
 const int CFG_MAX_LANGUAGE_CODE_LEN =    5;
 const int CFG_MAX_FIELDNAME_LEN     =  255;
 const int CFG_MAX_MISC_LEN          =  255;
+const int CFG_MAX_COLUMNS           =   64;
 
 class QColor; //lint !e763
 class QFont;  //lint !e763
@@ -68,7 +69,12 @@ class QFont;  //lint !e763
 typedef enum
 {
    FONTOBJECT_MENU=0,                 // must start with 0, as it is used for addressing an array
-   FONTOBJECT_DIALOGDEFAULT,
+   FONTOBJECT_TOOLBAR,
+   FONTOBJECT_TABLE,
+   FONTOBJECT_INFOFIELD,
+   FONTOBJECT_ACQUISITION_DIALOGS,
+   FONTOBJECT_MESSAGE_DIALOGS,
+   FONTOBJECT_DIALOG_DATA,
 
    FONTOBJECT_COUNT
 } t_CfgFontObject;
@@ -81,6 +87,7 @@ typedef enum
    COLOR_ADDITIONALSTATE3,
    COLOR_ADDITIONALSTATE4,
    COLOR_STATE_IDLE,
+   COLOR_STATE_QUEUED,
    COLOR_STATE_ACQUIRE,
    COLOR_STATE_ACQUIRE_PAUSED,
    COLOR_STATE_VERIFY,
@@ -88,6 +95,7 @@ typedef enum
    COLOR_STATE_CLEANUP,
    COLOR_STATE_FINISHED,
    COLOR_STATE_FINISHED_BADVERIFY,
+   COLOR_STATE_FINISHED_DUPLICATE_FAILED,
    COLOR_STATE_ABORTED_USER,
    COLOR_STATE_ABORTED_OTHER,
 
@@ -104,6 +112,15 @@ typedef enum
    SCANMETHOD_COUNT
 } t_CfgScanMethod;
 
+typedef enum
+{
+   EWFNAMING_OLD,
+   EWFNAMING_FTK
+} t_CfgEwfNaming;
+
+const int LIBEWF_COMPRESSION_EMPTY = 100; // Take care not to collide with LIBEWF definitions LIBEWF_COMPRESSION_FAST, LIBEWF_COMPRESSION_BEST etc.
+
+
 // ATTENTION: The data of the following structure is filled in by the CFG tool. As this
 // one doesn't know about 'bool' (it's written in ANSI C), bool MUST not be used here.
 // ------------------------------------------------------------------------------------
@@ -112,6 +129,7 @@ typedef enum
 typedef struct
 {
    char                 Language[CFG_MAX_LANGUAGE_LEN+1];
+   int                  CheckRootRights;
 
    t_CfgStartupSize     StartupSize;
    int                  StartupSizeManualX;
@@ -126,24 +144,39 @@ typedef struct
    t_CfgNumberStyle     NumberStyle;
    int                  ScreenRefreshInterval;
    int                  UseFileDialogFromQt;
-   
+   char                 UserFieldName[CFG_MAX_MISC_LEN+1];
+
+   int                  WarnAboutImageSize;
+   int                  WarnAboutSegmentFileCount;
+
    int                  AutoExit;
    int                  AutoExitCountdown;
 
    t_CfgScanMethod      DeviceScanMethod;
    int                  ScanInterval;
    char                 CommandGetSerialNumber[CFG_MAX_PATH_LEN+1];
+   int                  ForceCommandGetSerialNumber;
    char                 CommandGetAddStateInfo[CFG_MAX_PATH_LEN+1];
+   char                 CommandAcquisitionEnd [CFG_MAX_PATH_LEN+1];
    int                  QueryDeviceMediaInfo;
+   char                 RunStatsTemplateActive[CFG_MAX_PATH_LEN+1];
+   char                 RunStatsTemplateEnded [CFG_MAX_PATH_LEN+1];
+   char                 RunStatsOutput        [CFG_MAX_PATH_LEN+1];
+   int                  RunStatsInterval;
+
    int                  DirectIO;
 
    int                  DefaultFormat;
    int                  EwfFormat;
    int                  EwfCompression;
+   double               EwfCompressionThreshold;
+   t_CfgEwfNaming       EwfNaming;
+   int                  AffEnabled;
    int                  AffCompression;
    int                  AffMarkBadSectors;
    char                 SpecialFilenameChars[CFG_MAX_MISC_LEN+1];
    int                  CalcImageFileMD5;
+   int                  DuplicateImage;
    int                  AvoidEncaseProblems;
 
    int                  FifoBlockSizeDD;
@@ -153,6 +186,10 @@ typedef struct
    int                  FifoMemoryManager;
    int                  UseSeparateHashThread;
    int                  CompressionThreads;
+   int                  LimitJobs;
+   int                  JobMaxBadSectors;
+   int                  JobDisconnectTimeout;
+   int                  LocalHiddenDevicesUseRegExp;
 
    int                  SignalHandling;
    int                  WriteToDevNull;
@@ -178,6 +215,16 @@ typedef struct
    int             Italic;
 } t_CfgBuffFont, *t_pCfgBuffFont;
 
+typedef struct
+{
+   char Name[CFG_MAX_FIELDNAME_LEN+1];
+   int  Alignment;
+   int  MinWidth;
+   int  ShowInMainTable;
+   int  ShowInCloneTable;
+} t_CfgColumn, *t_pCfgColumn;
+typedef t_CfgColumn const *t_pcCfgColumn;
+
 typedef struct
 {
    t_CfgColor Color;
@@ -189,7 +236,7 @@ typedef struct
 typedef struct
 {
    char Device [CFG_MAX_PATH_LEN+1];
-} t_CfgBuffLocalDevice, *t_pCfgBuffLocalDevice;
+} t_CfgBuffLocalHiddenDevice, *t_pCfgBuffLocalHiddenDevice;
 
 typedef struct
 {
@@ -197,6 +244,12 @@ typedef struct
 } t_CfgBuffDeviceInfoCommand, *t_pCfgBuffDeviceInfoCommand;
 
 
+// Special constants
+// -----------------
+#define CONFIG_LIMITJOBS_OFF  -2
+#define CONFIG_LIMITJOBS_AUTO -1
+
+
 // DlgAcquire fields and rules
 // ---------------------------
 
@@ -207,22 +260,24 @@ typedef struct
 #define CFG_DLGACQUIRE_DST_FIELDID        "Dest"
 #define CFG_DLGACQUIRE_DIR_FIELDID        "Directory"
 
-#define CFG_DLGACQUIRE_SPLITFILESWITCH         QT_TRANSLATE_NOOP("t_DlgAcquire", "SplitFileSwitch"   )
-#define CFG_DLGACQUIRE_SPLITFILESIZE           QT_TRANSLATE_NOOP("t_DlgAcquire", "SplitFileSize"     )
-#define CFG_DLGACQUIRE_SPLITFILEUNIT           QT_TRANSLATE_NOOP("t_DlgAcquire", "SplitFileUnit"     )
-#define CFG_DLGACQUIRE_EWF_CASENUMBER          QT_TRANSLATE_NOOP("t_DlgAcquire", "EwfCaseNumber"     )
-#define CFG_DLGACQUIRE_EWF_EVIDENCENUMBER      QT_TRANSLATE_NOOP("t_DlgAcquire", "EwfEvidenceNumber" )
-#define CFG_DLGACQUIRE_EWF_EXAMINER            QT_TRANSLATE_NOOP("t_DlgAcquire", "EwfExaminer"       )
-#define CFG_DLGACQUIRE_EWF_DESCRIPTION         QT_TRANSLATE_NOOP("t_DlgAcquire", "EwfDescription"    )
-#define CFG_DLGACQUIRE_EWF_NOTES               QT_TRANSLATE_NOOP("t_DlgAcquire", "EwfNotes"          )
-#define CFG_DLGACQUIRE_DEST_IMAGEDIRECTORY     QT_TRANSLATE_NOOP("t_DlgAcquire", "DestImageDirectory")
-#define CFG_DLGACQUIRE_DEST_IMAGEFILENAME      QT_TRANSLATE_NOOP("t_DlgAcquire", "DestImageFilename" )
-#define CFG_DLGACQUIRE_DEST_INFODIRECTORY      QT_TRANSLATE_NOOP("t_DlgAcquire", "DestInfoDirectory" )
-#define CFG_DLGACQUIRE_DEST_INFOFILENAME       QT_TRANSLATE_NOOP("t_DlgAcquire", "DestInfoFilename"  )
-#define CFG_DLGACQUIRE_HASH_CALC_MD5           QT_TRANSLATE_NOOP("t_DlgAcquire", "HashCalcMD5"       )
-#define CFG_DLGACQUIRE_HASH_CALC_SHA256        QT_TRANSLATE_NOOP("t_DlgAcquire", "HashCalcSHA256"    )
-#define CFG_DLGACQUIRE_HASH_VERIFY_SRC         QT_TRANSLATE_NOOP("t_DlgAcquire", "HashVerifySrc"     )
-#define CFG_DLGACQUIRE_HASH_VERIFY_DST         QT_TRANSLATE_NOOP("t_DlgAcquire", "HashVerifyDst"     )
+#define CFG_DLGACQUIRE_SPLITFILESWITCH     QT_TRANSLATE_NOOP("t_DlgAcquire", "SplitFileSwitch"   )
+#define CFG_DLGACQUIRE_SPLITFILESIZE       QT_TRANSLATE_NOOP("t_DlgAcquire", "SplitFileSize"     )
+#define CFG_DLGACQUIRE_SPLITFILEUNIT       QT_TRANSLATE_NOOP("t_DlgAcquire", "SplitFileUnit"     )
+#define CFG_DLGACQUIRE_EWF_CASENUMBER      QT_TRANSLATE_NOOP("t_DlgAcquire", "EwfCaseNumber"     )
+#define CFG_DLGACQUIRE_EWF_EVIDENCENUMBER  QT_TRANSLATE_NOOP("t_DlgAcquire", "EwfEvidenceNumber" )
+#define CFG_DLGACQUIRE_EWF_EXAMINER        QT_TRANSLATE_NOOP("t_DlgAcquire", "EwfExaminer"       )
+#define CFG_DLGACQUIRE_EWF_DESCRIPTION     QT_TRANSLATE_NOOP("t_DlgAcquire", "EwfDescription"    )
+#define CFG_DLGACQUIRE_EWF_NOTES           QT_TRANSLATE_NOOP("t_DlgAcquire", "EwfNotes"          )
+#define CFG_DLGACQUIRE_USERFIELD                                             "UserField"
+#define CFG_DLGACQUIRE_DEST_IMAGEDIRECTORY QT_TRANSLATE_NOOP("t_DlgAcquire", "DestImageDirectory")
+#define CFG_DLGACQUIRE_DEST_IMAGEFILENAME  QT_TRANSLATE_NOOP("t_DlgAcquire", "DestImageFilename" )
+#define CFG_DLGACQUIRE_DEST_INFODIRECTORY  QT_TRANSLATE_NOOP("t_DlgAcquire", "DestInfoDirectory" )
+#define CFG_DLGACQUIRE_DEST_INFOFILENAME   QT_TRANSLATE_NOOP("t_DlgAcquire", "DestInfoFilename"  )
+#define CFG_DLGACQUIRE_HASH_CALC_MD5       QT_TRANSLATE_NOOP("t_DlgAcquire", "HashCalcMD5"       )
+#define CFG_DLGACQUIRE_HASH_CALC_SHA1      QT_TRANSLATE_NOOP("t_DlgAcquire", "HashCalcSHA1"      )
+#define CFG_DLGACQUIRE_HASH_CALC_SHA256    QT_TRANSLATE_NOOP("t_DlgAcquire", "HashCalcSHA256"    )
+#define CFG_DLGACQUIRE_HASH_VERIFY_SRC     QT_TRANSLATE_NOOP("t_DlgAcquire", "HashVerifySrc"     )
+#define CFG_DLGACQUIRE_HASH_VERIFY_DST     QT_TRANSLATE_NOOP("t_DlgAcquire", "HashVerifyDst"     )
 
 typedef struct
 {
@@ -278,10 +333,16 @@ typedef t_CfgDlgAcquireRules       *t_pCfgDlgAcquireRules;
 QFont        *CfgGetpFont    (t_CfgFontObject Object);
 const QColor *CfgGetpColor   (t_CfgColor      Color );
 APIRET CfgGetLocalDevices        (QStringList **ppLocalDevices);
+APIRET CfgGetHiddenDevices       (QStringList **ppLocalDevices);
 APIRET CfgGetDeviceInfoCommands  (QStringList **ppDeviceInfoCommands);
 APIRET CfgGetDlgAcquireFields    (t_pCfgDlgAcquireFields *ppDlgAcquireFields);
 APIRET CfgGetDlgAcquireRules     (t_pCfgDlgAcquireRules  *ppDlgAcquireRules);
 
+QString CfgGetHostName   (void);
+QString CfgGetMacAddress (void);
+
+int           CfgGetColumnCount (void);
+t_pcCfgColumn CfgGetColumn      (int i);
 
 APIRET CfgInit        (void);
 APIRET CfgDeInit      (void);
@@ -303,6 +364,7 @@ void CfgFontPrint (void);
 enum
 {
    ERROR_CFG_ONLY_TEMPLATE_GENERATED = ERROR_BASE_CFG + 1,
+   ERROR_CFG_INCOHERENCY_DETECTED
 };
 
 
