@@ -9,7 +9,7 @@
 //  Module:         Directory selection dialog
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -26,10 +26,9 @@
 // You should have received a copy of the GNU General Public License
 // along with guymager. If not, see <http://www.gnu.org/licenses/>.
 
-#include <QtGui>
-
-#include "common.h"
 #include "dlgmessage.h"
+#include "config.h"
+#include "qtutil.h"
 #include "dlgdirsel.h"
 #include "dlgdirsel_private.h"
 
@@ -70,6 +69,8 @@ t_DlgDirSel::t_DlgDirSel (QString *pPath, QWidget *pParent, Qt::WFlags Flags)
       CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_DLGDIRSEL_CONSTRUCTOR_NOT_SUPPORTED))
    }
 
+   QTUTIL_SET_FONT (this, FONTOBJECT_MESSAGE_DIALOGS)
+
    pOwn         = new t_DlgDirSelLocal;
    pOwn->pTree  = new t_TreeView (this);
    pOwn->pModel = new QDirModel  (this);
