@@ -9,7 +9,7 @@
 //  Module:         The central queues for pipelined data processing
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -39,18 +39,6 @@
 #include "fifo.h"
 #include "stdlib.h"
 
-class t_FifoStdLocal
-{
-   public:
-      QQueue<t_pFifoBlock>  Queue;
-      int                   MaxBlocks;
-      QSemaphore          *pSemEmptyBlocks;
-      QSemaphore          *pSemUsedBlocks;
-      QMutex              *pMutexQueue;
-      t_pFifoMemory        pFifoMemory;
-};
-
-
 static QMutex  MutexGlobal;
 static quint64 Allocs   =0;
 static quint64 Frees    =0;
@@ -186,11 +174,9 @@ APIRET FifoMemoryFree (t_pFifoMemory *ppFifoMemory)
 }
 
 // ----------------------------
-//            FIFO
+//     FIFO block handling
 // ----------------------------
 
-// We use ANSI C malloc/free for the FIFO blocks, as we want to be sure that it's fast
-
 APIRET t_Fifo::Create (t_pFifoMemory pFifoMemory, t_pFifoBlock &pBlock, unsigned int AllocSize)
 {
    if (CONFIG(FifoMemoryManager))
@@ -200,19 +186,21 @@ APIRET t_Fifo::Create (t_pFifoMemory pFifoMemory, t_pFifoBlock &pBlock, unsigned
    if (pBlock == NULL)
       CHK_CONST (ERROR_FIFO_MALLOC_FAILED)
 //   memset (pBlock, 0xAA, sizeof(t_FifoBlock) + BufferSize);
+
+   memset (pBlock, 0, sizeof(t_FifoBlock));
    pBlock->MagicStart         = FIFO_MAGIC_START;
    pBlock->BufferSize         = AllocSize - sizeof (t_FifoBlock);
-   pBlock->DataSize           = 0;
-   pBlock->LastBlock          = false;
+//   pBlock->DataSize           = 0;
+//   pBlock->LastBlock          = false;
 
-   pBlock->EwfPreprocessed    = false;
-   pBlock->EwfDataSize        = 0;
-   pBlock->EwfCompressionUsed = 0;
-   pBlock->EwfChunkCRC        = 0;
-   pBlock->EwfWriteCRC        = 0;
+//   pBlock->EwfPreprocessed    = false;
+//   pBlock->EwfDataSize        = 0;
+//   pBlock->EwfCompressionUsed = 0;
+//   pBlock->EwfChunkCRC        = 0;
+//   pBlock->EwfWriteCRC        = 0;
 
-   pBlock->pAaffPreprocess    = NULL;
-   pBlock->pAewfPreprocess    = NULL;
+//   pBlock->pAaffPreprocess    = NULL;
+//   pBlock->pAewfPreprocess    = NULL;
 
    pBlock->Nr                 = FIFO_NR_NOTSET;
 
@@ -228,6 +216,31 @@ APIRET t_Fifo::Create (t_pFifoMemory pFifoMemory, t_pFifoBlock &pBlock, unsigned
    return NO_ERROR;
 }
 
+APIRET t_Fifo::Duplicate(t_pFifoMemory pFifoMemory, t_pFifoBlock pBlockSrc, t_pFifoBlock &pBlockDst)
+{
+   pBlockDst = NULL;
+   if (pBlockSrc == NULL)  // Dummy block
+      return NO_ERROR;
+
+   unsigned int AllocSize = pBlockSrc->BufferSize + sizeof(t_FifoBlock);
+
+   if (CONFIG(FifoMemoryManager))
+        pBlockDst = (t_pFifoBlock) pFifoMemory->GetBlock (AllocSize);
+   else pBlockDst = (t_pFifoBlock) UTIL_MEM_ALLOC        (AllocSize);
+
+   if (pBlockDst == NULL)
+      CHK_CONST (ERROR_FIFO_MALLOC_FAILED)
+
+   memcpy (pBlockDst, pBlockSrc, AllocSize);
+
+   MutexGlobal.lock();
+   Allocs++;
+   Allocated += AllocSize;
+   MutexGlobal.unlock();
+
+   return NO_ERROR;
+}
+
 //lint -save -e661 -e826 -esym(613,pBlock) Possible access of out-of-bounds pointer, suspicious ptr to ptr conversion, possible use of NULL pointer
 APIRET t_Fifo::Destroy (t_pFifoMemory pFifoMemory, t_pFifoBlock &pBlock)
 {
@@ -277,30 +290,49 @@ void t_Fifo::LogBlock (t_pFifoMemory pFifoMemory, t_pFifoBlock pBlock, char Mark
 }
 
 
+// ----------------------------
+//        Standard FIFO
+// ----------------------------
+
+class t_FifoStdLocal
+{
+   public:
+      QQueue<t_pFifoBlock>  Queue;
+      int                   MaxBlocks;
+      QSemaphore          *pSemEmptyBlocks;
+      QSemaphore          *pSemUsedBlocks;
+      QMutex              *pMutexQueue;
+      t_pFifoMemory        pFifoMemory;
+      t_pFifoStd           pFifoDup;
+};
+
 t_FifoStd::t_FifoStd (t_pFifoMemory pFifoMemory, int MaxBlocks)
 {
    static bool Initialised = false;
 
    if (!Initialised)
    {
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MALLOC_FAILED           ))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_DOUBLE_FREE             ))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_EMPTY                   ))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_END_CORRUPTED           ))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_START_CORRUPTED         ))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MEMORY_UNDERUN          ))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MEMORY_ALLOC_FAILED     ))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MEMORY_INVALID_BLOCKSIZE))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MEMORY_ALREADY_ALLOCATED))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MEMORY_NOT_ALLOCATED    ))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_POINTER_TOO_LOW         ))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_POINTER_NOT_ON_BOUNDARY ))
-      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_POINTER_TOO_HIGH        ))
       Initialised = true;
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MALLOC_FAILED            ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_DOUBLE_FREE              ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_EMPTY                    ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_END_CORRUPTED            ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_START_CORRUPTED          ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MEMORY_UNDERUN           ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MEMORY_ALLOC_FAILED      ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MEMORY_INVALID_BLOCKSIZE ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MEMORY_ALREADY_ALLOCATED ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_MEMORY_NOT_ALLOCATED     ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_POINTER_TOO_LOW          ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_POINTER_NOT_ON_BOUNDARY  ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_POINTER_TOO_HIGH         ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_ALREADY_DUPLICATED       ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_FIFO_DUPLICATION_NOT_SUPPORTED))
    }
    pOwn = new t_FifoStdLocal;
    pOwn->pFifoMemory = pFifoMemory;
    pOwn->MaxBlocks   = MaxBlocks;
+   pOwn->pFifoDup    = NULL;
 
    pOwn->pSemEmptyBlocks = new QSemaphore (MaxBlocks);
    pOwn->pSemUsedBlocks  = new QSemaphore (0);
@@ -318,14 +350,36 @@ t_FifoStd::~t_FifoStd ()
          CHK_EXIT (Destroy (pOwn->pFifoMemory, pBlock))
    }
 
+   if (pOwn->pFifoDup)
+      delete pOwn->pFifoDup;
    delete pOwn->pSemEmptyBlocks;
    delete pOwn->pSemUsedBlocks;
    delete pOwn->pMutexQueue;
    delete pOwn;
 }
 
+APIRET t_FifoStd::DupFifo (t_pFifo *ppFifo)
+{
+   *ppFifo = NULL;
+
+   if (pOwn->pFifoDup)
+      return ERROR_FIFO_ALREADY_DUPLICATED;
+
+   pOwn->pFifoDup = new t_FifoStd (pOwn->pFifoMemory, pOwn->MaxBlocks);
+   *ppFifo = pOwn->pFifoDup;
+
+   return NO_ERROR;
+}
+
 APIRET t_FifoStd::Insert (t_pFifoBlock pBlock)
 {
+   if (pOwn->pFifoDup)                   // Important: Duplicate before inserting into this fifo, or else the block
+   {                                     // could already have been taken out of the Fifo and destroyed.
+      t_pFifoBlock pBlockDup;
+      CHK (Duplicate (pOwn->pFifoMemory, pBlock, pBlockDup));
+      CHK (pOwn->pFifoDup->Insert(pBlockDup))
+   }
+
    pOwn->pSemEmptyBlocks->acquire();
    pOwn->pMutexQueue->lock();
    pOwn->Queue.enqueue (pBlock);
@@ -365,9 +419,17 @@ APIRET t_FifoStd::Count (int &Cnt)
 APIRET t_FifoStd::Usage (int &Percent)
 {
    int Cnt;
+   int Max = pOwn->MaxBlocks;
 
    CHK (Count(Cnt))
-   Percent = (100LL * Cnt) / pOwn->MaxBlocks;
+   if (pOwn->pFifoDup)
+   {
+      int CntDup;
+      CHK (pOwn->pFifoDup->Count(CntDup))
+      Cnt += CntDup;
+      Max *= 2;
+   }
+   Percent = (100LL * Cnt) / Max;
    return NO_ERROR;
 }
 
@@ -402,6 +464,9 @@ APIRET t_FifoStd::WakeWaitingThreads (void)
       pOwn->pSemEmptyBlocks->release();
    }
 
+   if (pOwn->pFifoDup)
+      CHK (pOwn->pFifoDup->WakeWaitingThreads())
+
    return NO_ERROR;
 }
 
@@ -419,6 +484,9 @@ class t_FifoCompressLocal   // used as well for t_FifoCompressOut
       int           TotalMaxBlocks;    // Sum of MaxBlocks of all FIFOs
       QMutex      *pMutexQueue;
       int           NextSubFifoNr;
+      t_pFifo      pFifoDup;
+      bool          Dup;               // Set if this is a duplicated FIFO (i.e. which refers to the duplicated
+                                       // subfifos of another t_FifoCompressOut in ints ppFifoArr).
 };
 
 static APIRET FifoCompressInit (t_FifoCompressLocal **ppOwn, t_pFifoMemory pFifoMemory, int SubFifos, int MaxBlocks)
@@ -433,15 +501,19 @@ static APIRET FifoCompressInit (t_FifoCompressLocal **ppOwn, t_pFifoMemory pFifo
 
    (*ppOwn)->NextSubFifoNr = 0;
    (*ppOwn)->pMutexQueue   = new QMutex();
+   (*ppOwn)->pFifoDup      = NULL;
+   (*ppOwn)->Dup           = false;
 
    return NO_ERROR;
 }
 
-static APIRET FifoCompressDeInit (t_FifoCompressLocal *pOwn)
+static APIRET FifoCompressDeInit (t_FifoCompressLocal *pOwn, bool RemoveSubFifos=true)
 {
-   for (int i=0; i<pOwn->FifoArrLen; i++)
-      delete (pOwn->ppFifoArr[i]);
-
+   if (RemoveSubFifos)
+   {
+      for (int i=0; i<pOwn->FifoArrLen; i++)
+         delete (pOwn->ppFifoArr[i]);
+   }
    UTIL_MEM_FREE (pOwn->ppFifoArr);
    delete pOwn->pMutexQueue;
    delete pOwn;
@@ -465,6 +537,11 @@ t_FifoCompressIn::~t_FifoCompressIn ()
    CHK_EXIT (FifoCompressDeInit (pOwn))
 }
 
+APIRET t_FifoCompressIn::DupFifo (t_pFifo */*ppFifo*/)
+{
+   return ERROR_FIFO_DUPLICATION_NOT_SUPPORTED;
+}
+
 APIRET t_FifoCompressIn::GetSubFifo (int SubFifoNr, t_pFifoStd &pFifo)
 {
    pFifo = pOwn->ppFifoArr[SubFifoNr];
@@ -546,12 +623,28 @@ APIRET t_FifoCompressIn::WakeWaitingThreads (void)
 //                                       FIFO Compress Out
 // ----------------------------------------------------------------------------------------
 
-t_FifoCompressOut::t_FifoCompressOut (void)
+t_FifoCompressOut::t_FifoCompressOut ()
 {
    pOwn = NULL;   //lint -esym(613,t_FifoCompressOut::pOwn)  Prevent lint from telling us about possible null pointers in the following code
    CHK_EXIT (ERROR_FIFO_CONSTRUCTOR_NOT_SUPPORTED)
 }
 
+t_FifoCompressOut::t_FifoCompressOut (t_pFifoCompressOut pFifoSrc)
+{
+   pOwn = new t_FifoCompressLocal;
+   pOwn->TotalMaxBlocks = pFifoSrc->pOwn->TotalMaxBlocks;
+   pOwn->FifoArrLen     = pFifoSrc->pOwn->FifoArrLen;
+   pOwn->ppFifoArr      = (t_pFifoStd *) UTIL_MEM_ALLOC ((size_t)pOwn->FifoArrLen * sizeof(t_pFifoStd));
+
+   for (int i=0; i<pOwn->FifoArrLen; i++)
+      pOwn->ppFifoArr[i] = NULL;
+
+   pOwn->NextSubFifoNr = 0;
+   pOwn->pMutexQueue   = new QMutex();
+   pOwn->pFifoDup      = NULL;
+   pOwn->Dup           = true;
+}
+
 t_FifoCompressOut::t_FifoCompressOut (t_pFifoMemory pFifoMemory, int SubFifos, int MaxBlocks)
 {
    CHK_EXIT (FifoCompressInit (&pOwn, pFifoMemory, SubFifos, MaxBlocks))
@@ -559,7 +652,28 @@ t_FifoCompressOut::t_FifoCompressOut (t_pFifoMemory pFifoMemory, int SubFifos, i
 
 t_FifoCompressOut::~t_FifoCompressOut ()
 {
-   CHK_EXIT (FifoCompressDeInit (pOwn))
+   if (pOwn->pFifoDup)
+      delete (pOwn->pFifoDup);
+   CHK_EXIT (FifoCompressDeInit (pOwn, !pOwn->Dup))
+}
+
+APIRET t_FifoCompressOut::DupFifo (t_pFifo *ppFifo)
+{
+   t_pFifoCompressOut   pFifoCompressOutDup;
+   t_pFifo              pSubFifoDup;
+   t_FifoCompressLocal *pDupOwn;
+
+   pFifoCompressOutDup = new t_FifoCompressOut (this);
+   pDupOwn = pFifoCompressOutDup->pOwn;
+   for (int i=0; i<pOwn->FifoArrLen; i++)
+   {
+      pOwn->ppFifoArr[i]->DupFifo (&pSubFifoDup);
+      pDupOwn->ppFifoArr[i] = (t_pFifoStd) pSubFifoDup;
+   }
+
+   *ppFifo = pFifoCompressOutDup;
+
+   return NO_ERROR;
 }
 
 APIRET t_FifoCompressOut::GetSubFifo (int SubFifoNr, t_pFifoStd &pFifo)
@@ -625,9 +739,17 @@ APIRET t_FifoCompressOut::Count (int &Cnt)
 APIRET t_FifoCompressOut::Usage (int &Percent)
 {
    int Cnt;
+   int Max = pOwn->TotalMaxBlocks;
 
    CHK (Count(Cnt))
-   Percent = (100LL * Cnt) / pOwn->TotalMaxBlocks;
+   if (pOwn->pFifoDup)
+   {
+      int CntDup;
+      CHK (pOwn->pFifoDup->Count(CntDup))
+      Cnt += CntDup;
+      Max *= 2;
+   }
+   Percent = (100LL * Cnt) / Max;
    Percent = GETMIN (Percent, 100); // In some cases, the usage could be above 100% (see remarks for t_FifoCompressIn::Count); so we limit to 100
    return NO_ERROR;
 }
