@@ -9,7 +9,7 @@
 //  Module:         The info field area
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -32,6 +32,8 @@
 
 #include "common.h"
 #include "main.h"
+#include "config.h"
+#include "qtutil.h"
 #include "infofield.h"
 
 class t_InfoFieldLocal
@@ -49,33 +51,19 @@ t_InfoField::t_InfoField ()
 t_InfoField::t_InfoField (QWidget *pParent)
    :QFrame (pParent)
 {
-   QString LabelParams;
-   
    CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_INFOFIELD_CONSTRUCTOR_NOT_SUPPORTED))
 
    setFrameStyle ((int)QFrame::Panel | (int)QFrame::Sunken);
    pOwn = new t_InfoFieldLocal;
 
    QHBoxLayout *pLayout = new QHBoxLayout (this);
-   pOwn->pLabelParams = new QLabel (this);
    pOwn->pLabelValues = new QLabel (this);
+   pOwn->pLabelParams = new QLabel (this);
+   QTUTIL_SET_FONT (pOwn->pLabelValues, FONTOBJECT_INFOFIELD)
+   QTUTIL_SET_FONT (pOwn->pLabelParams, FONTOBJECT_INFOFIELD)
    pLayout->addWidget (pOwn->pLabelParams);
    pLayout->addWidget (pOwn->pLabelValues);
    pLayout->addStretch();
-   
-   if (CONFIG (CommandGetAddStateInfo)[0])
-      LabelParams = tr("Additional state info") + "\n";
-      
-   LabelParams +=       tr("Size")
-               + "\n" + tr("Sector size")
-               + "\n" + tr("Image file")
-               + "\n" + tr("Info file")
-               + "\n" + tr("Current speed")
-               + "\n" + tr("Started", "Start timestamp and running time")
-               + "\n" + tr("Hash calculation")
-               + "\n" + tr("Source verification")
-               + "\n" + tr("Image verification");
-   pOwn->pLabelParams->setText (LabelParams);
 }
 
 t_InfoField::~t_InfoField ()
@@ -85,10 +73,45 @@ t_InfoField::~t_InfoField ()
    delete pOwn;
 }
 
+QString t_InfoField::GetHashText (bool MD5, bool SHA1, bool SHA256)
+{
+   QString Str;
+
+   if      (!MD5 && !SHA1 && !SHA256) Str = t_InfoField::tr("off"          , "Hash calculation is off");
+   else if (!MD5 && !SHA1 &&  SHA256) Str = t_InfoField::tr("%1"           , "Show which hash values are in use, % value wll be replaced by MD5, SHA-1, ...") .arg("SHA-256");
+   else if (!MD5 &&  SHA1 && !SHA256) Str = t_InfoField::tr("%1"           , "Show which hash values are in use, % value wll be replaced by MD5, SHA-1, ...") .arg("SHA-1");
+   else if (!MD5 &&  SHA1 &&  SHA256) Str = t_InfoField::tr("%1 and %2"    , "Show which hash values are in use, % value wll be replaced by MD5, SHA-1, ...") .arg("SHA-1") .arg("SHA-256");
+   else if ( MD5 && !SHA1 && !SHA256) Str = t_InfoField::tr("%1"           , "Show which hash values are in use, % value wll be replaced by MD5, SHA-1, ...") .arg("MD5");
+   else if ( MD5 && !SHA1 &&  SHA256) Str = t_InfoField::tr("%1 and %2"    , "Show which hash values are in use, % value wll be replaced by MD5, SHA-1, ...") .arg("MD5") .arg("SHA-256");
+   else if ( MD5 &&  SHA1 && !SHA256) Str = t_InfoField::tr("%1 and %2"    , "Show which hash values are in use, % value wll be replaced by MD5, SHA-1, ...") .arg("MD5") .arg("SHA-1");
+   else                               Str = t_InfoField::tr("%1, %2 and %3", "Show which hash values are in use, % value wll be replaced by MD5, SHA-1, ...") .arg("MD5") .arg("SHA-1") .arg("SHA-256");
+   return Str;
+}
+
 void t_InfoField::SlotShowInfo (t_pDevice pDevice)
 {
    QString StrValue, StrSpeed, Format;
    quint64 Size;
+   QString LabelParams;
+   bool    DuplicateImage=false;
+
+   if (pDevice)
+      if (pDevice->Duplicate)
+         DuplicateImage = true;
+
+   if (CONFIG (CommandGetAddStateInfo)[0])
+      LabelParams = tr("Additional state info") + "\n";
+
+   LabelParams +=       tr("Size")
+               +  "\n" + tr("Sector size")
+               +  "\n" + tr("Image file");  if (DuplicateImage) LabelParams += "\n";
+   LabelParams += "\n" + tr("Info file" );  if (DuplicateImage) LabelParams += "\n";
+   LabelParams += "\n" + tr("Current speed")
+               + "\n" + tr("Started", "Start timestamp and running time")
+               + "\n" + tr("Hash calculation")
+               + "\n" + tr("Source verification")
+               + "\n" + tr("Image verification");
+   pOwn->pLabelParams->setText (LabelParams);
 
    if (pDevice)
    {
@@ -96,7 +119,7 @@ void t_InfoField::SlotShowInfo (t_pDevice pDevice)
       // ---------------------
       if (CONFIG (CommandGetAddStateInfo)[0])
          StrValue = pDevice->AddStateInfo.Info + "\n";
-         
+
       // Size
       // ----
       Size = t_Device::GetSize (pDevice).toULongLong();
@@ -114,16 +137,22 @@ void t_InfoField::SlotShowInfo (t_pDevice pDevice)
 
       // Image/Info file
       // ---------------
-      if (pDevice->Acquisition.Format == t_File::NotSet)
+      QString ImagePaths;
+      QString InfoPaths;
+
+      if (pDevice->Acquisition1.Format != t_File::NotSet)
       {
-         StrValue += "\n\n";
-      }
-      else
-      {
-         CHK_EXIT (t_File::GetFormatExtension   (pDevice->Acquisition.Format, pDevice->Acquisition.Clone, 0, NULL, &Format))
-         StrValue += "\n" + pDevice->Acquisition.ImagePath + pDevice->Acquisition.ImageFilename + QSTR_TO_PSZ(Format);
-         StrValue += "\n" + pDevice->Acquisition.InfoPath  + pDevice->Acquisition.InfoFilename  + t_File::pExtensionInfo;
+         CHK_EXIT (t_File::GetFormatExtension   (pDevice->Acquisition1.Format, pDevice->Acquisition1.Clone, 0, NULL, &Format))
+         ImagePaths = pDevice->Acquisition1.ImagePath + pDevice->Acquisition1.ImageFilename + QSTR_TO_PSZ(Format);
+         InfoPaths  = pDevice->Acquisition1.InfoPath  + pDevice->Acquisition1.InfoFilename  + t_File::pExtensionInfo;
+         if (DuplicateImage)
+         {
+            ImagePaths += "\n" + pDevice->Acquisition2.ImagePath + pDevice->Acquisition2.ImageFilename + QSTR_TO_PSZ(Format);
+            InfoPaths  += "\n" + pDevice->Acquisition2.InfoPath  + pDevice->Acquisition2.InfoFilename  + t_File::pExtensionInfo;
+         }
       }
+      StrValue += "\n" + ImagePaths;
+      StrValue += "\n" + InfoPaths;
 
       // Speed
       // -----
@@ -134,7 +163,7 @@ void t_InfoField::SlotShowInfo (t_pDevice pDevice)
 
       // Running time
       // ------------
-      if (pDevice->Acquisition.Format == t_File::NotSet)
+      if (pDevice->Acquisition1.Format == t_File::NotSet)
       {
          StrValue += "\n";
       }
@@ -144,10 +173,10 @@ void t_InfoField::SlotShowInfo (t_pDevice pDevice)
 
          StrValue += "\n" + pDevice->StartTimestamp.toString ("d. MMMM hh:mm:ss");
 
-         if ((pDevice->State == t_Device::Acquire      ) ||  // Don't display anything if no acquisition is running
-             (pDevice->State == t_Device::AcquirePaused) ||
-             (pDevice->State == t_Device::Verify       ) ||
-             (pDevice->State == t_Device::VerifyPaused ))
+         if ((pDevice->GetState() == t_Device::Acquire      ) ||  // Don't display anything if no acquisition is running
+             (pDevice->GetState() == t_Device::AcquirePaused) ||
+             (pDevice->GetState() == t_Device::Verify       ) ||
+             (pDevice->GetState() == t_Device::VerifyPaused ))
               Seconds = pDevice->StartTimestamp.secsTo (QDateTime::currentDateTime());
          else Seconds = pDevice->StartTimestamp.secsTo (pDevice->StopTimestamp);
          Hours    = Seconds / SECONDS_PER_HOUR  ; Seconds -= Hours   * SECONDS_PER_HOUR;
@@ -160,19 +189,15 @@ void t_InfoField::SlotShowInfo (t_pDevice pDevice)
       // ----
       StrValue += "\n";
       if (!pDevice->StartTimestamp.isNull())
-      {   
-         if     ((pDevice->Acquisition.CalcMD5) &&
-                 (pDevice->Acquisition.CalcSHA256)) StrValue += tr("MD5 and SHA-256" , "Both hashes are calculated");
-         else if (pDevice->Acquisition.CalcMD5)     StrValue += tr("MD5"    );
-         else if (pDevice->Acquisition.CalcSHA256)  StrValue += tr("SHA-256");
-         else                                       StrValue += tr("off", "Hash calculation is off");
-      }
+         StrValue += t_InfoField::GetHashText (pDevice->Acquisition1.CalcMD5,
+                                               pDevice->Acquisition1.CalcSHA1,
+                                               pDevice->Acquisition1.CalcSHA256);
       // Source verification
       // -------------------
       StrValue += "\n";
       if (!pDevice->StartTimestamp.isNull())
-      {   
-         if (pDevice->Acquisition.VerifySrc)
+      {
+         if (pDevice->Acquisition1.VerifySrc)
               StrValue += tr("on" , "Display that verification is on");
          else StrValue += tr("off", "Display that verification is off");
       }
@@ -180,10 +205,17 @@ void t_InfoField::SlotShowInfo (t_pDevice pDevice)
       // -------------------
       StrValue += "\n";
       if (!pDevice->StartTimestamp.isNull())
-      {   
-         if (pDevice->Acquisition.VerifyDst)
+      {
+         if (pDevice->Acquisition1.VerifyDst)
               StrValue += tr("on" , "Display that verification is on");
          else StrValue += tr("off", "Display that verification is off");
+         if (DuplicateImage)
+         {
+            StrValue += " | ";
+            if (pDevice->Acquisition2.VerifyDst)
+                 StrValue += tr("on" , "Display that verification is on");
+            else StrValue += tr("off", "Display that verification is off");
+         }
       }
 }
    pOwn->pLabelValues->setText (StrValue);
