@@ -10,7 +10,7 @@
 //                  in t_Table.
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -85,16 +85,18 @@ void t_ItemDelegate::PaintDefaults (QPainter *pPainter, const QStyleOptionViewIt
 
 void t_ItemDelegate::PaintProgress (QPainter *pPainter, const QStyleOptionViewItem &Option, const QModelIndex &Index) const
 {
-   QRect  BarRect;
-   QColor ColorPen;
-   double ProgressFactor;
+   t_pDevice pDev;
+   QRect      BarRect;
+   QColor     ColorPen;
+   double     ProgressFactor;
 
    pPainter->save();
    PaintDefaults (pPainter, Option, Index, ColorPen);
 
-   ProgressFactor = Index.data (Qt::DisplayRole).toDouble();
-   if (ProgressFactor >= 0.0)
+   pDev = (t_pDevice) Index.data(DeviceRole).value<void *>();
+   if (!pDev->StartTimestamp.isNull())
    {
+      ProgressFactor = Index.data (Qt::DisplayRole).toDouble();
       BarRect = Option.rect;
       BarRect.adjust (10, 20, -10, -4);
       pPainter->setBrush (Qt::NoBrush);
@@ -106,7 +108,6 @@ void t_ItemDelegate::PaintProgress (QPainter *pPainter, const QStyleOptionViewIt
       pPainter->setPen   (Qt::NoPen);
       BarRect.setWidth ((int) (BarRect.width() * ProgressFactor));
       pPainter->drawRect (BarRect);
-
    }
    pPainter->restore  ();
 }
@@ -138,20 +139,27 @@ void t_ItemDelegate::PaintState (QPainter *pPainter, const QStyleOptionViewItem
 
    pDev = (t_pDevice) Index.data(DeviceRole).value<void *>();
 
-   switch (pDev->State)
+   switch (pDev->GetState())
    {
       case t_Device::Idle         : CircleColor = COLOR_STATE_IDLE;           break;
+      case t_Device::Queued       : CircleColor = COLOR_STATE_QUEUED;         break;
       case t_Device::Acquire      : CircleColor = COLOR_STATE_ACQUIRE;        break;
       case t_Device::AcquirePaused: CircleColor = COLOR_STATE_ACQUIRE_PAUSED; break;
       case t_Device::Verify       : CircleColor = COLOR_STATE_VERIFY;         break;
       case t_Device::VerifyPaused : CircleColor = COLOR_STATE_VERIFY_PAUSED;  break;
       case t_Device::Cleanup      : CircleColor = COLOR_STATE_CLEANUP;        break;
-      case t_Device::Finished     : if ((pDev->Acquisition.VerifySrc && (!HashMD5Match (&pDev->MD5Digest, &pDev->MD5DigestVerifySrc) || !HashSHA256Match(&pDev->SHA256Digest, &pDev->SHA256DigestVerifySrc))) ||
-                                        (pDev->Acquisition.VerifyDst && (!HashMD5Match (&pDev->MD5Digest, &pDev->MD5DigestVerifyDst) || !HashSHA256Match(&pDev->SHA256Digest, &pDev->SHA256DigestVerifyDst))))
-                                         CircleColor = COLOR_STATE_FINISHED_BADVERIFY;
-                                    else CircleColor = COLOR_STATE_FINISHED;
+      case t_Device::Finished     : if (pDev->HashMatch())
+                                    {
+                                       if (pDev->Duplicate && (pDev->Error.Get(1) || pDev->Error.Get(2)))
+                                            CircleColor = COLOR_STATE_FINISHED_DUPLICATE_FAILED;
+                                       else CircleColor = COLOR_STATE_FINISHED;
+                                    }
+                                    else
+                                    {
+                                       CircleColor = COLOR_STATE_FINISHED_BADVERIFY;
+                                    }
                                     break;
-      case t_Device::Aborted      : if (pDev->AbortReason==t_Device::UserRequest)
+      case t_Device::Aborted      : if (pDev->Error.Get()==t_Device::t_Error::UserRequest)
                                          CircleColor = COLOR_STATE_ABORTED_USER;
                                     else CircleColor = COLOR_STATE_ABORTED_OTHER;
                                     break;
