@@ -11,7 +11,7 @@
 //                  deinitialisation.
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -33,7 +33,7 @@
 #include <signal.h>
 #include <locale.h>
 #include <pwd.h>
-// #include "proc/sysinfo.h"  // Required in order to get the amount of memory installed
+#include <unistd.h>
 
 #include <QApplication>
 #include <QtGui>
@@ -55,6 +55,7 @@
 #include "hash.h"
 #include "aaff.h"
 #include "aewf.h"
+#include "dlgmessage.h"
 #include "mainwindow.h"
 #include "main.h"
 
@@ -69,14 +70,15 @@
 //              Constants
 // ------------------------------------
 
-static const int MAIN_MAX_ERROR_CODES       = 1000;
-static const int MAIN_SHOW_SPLASH_TIME      =  500;  // ms
-static const int MAIN_PROCESSEVENTS_MAXTIME = 1000;
+static const int MAIN_MAX_ERROR_CODES        = 1000;
+static const int MAIN_SHOW_SPLASH_TIME_EVENT =  300;  // ms
+static const int MAIN_PROCESSEVENTS_MAXTIME  = 1000;
 
 enum
 {
    ERROR_MAIN_UNKNOWN_STARTUP_SIZE = ERROR_BASE_MAIN + 1,
    ERROR_MAIN_INVALID_WINDOW_SIZE,
+   ERROR_MAIN_INVALID_NUMBERSTYLE,
    ERROR_MAIN_LIBEWF_VERSION_MISMATCH
 };
 
@@ -199,20 +201,54 @@ static APIRET MainWindowAdjustSize (t_MainWindow *pMainWindow)
 static void MainCfgLog (t_pcchar pFileName, t_pcchar pFunction, t_int LineNr, t_pcchar pFormat, va_list pArguments)
 {
    static bool ConsoleLog = false;
+   va_list     pArgumentsCopy;
 
    if (!ConsoleLog)
         ConsoleLog = (strstr (pFormat, "Configuration error"       ) != NULL);
    else ConsoleLog = (strstr (pFormat, "TOOLCFG_ERROR_CONFIG_ERROR") == NULL);
 
+   __va_copy (pArgumentsCopy, pArguments);
    if (ConsoleLog)
    {
       printf ("\n");
       vprintf (pFormat, pArguments);
    }
-
-   t_Log::vEntryInfo (pFileName, pFunction, LineNr, pFormat, pArguments);
+   t_Log::vEntryInfo (pFileName, pFunction, LineNr, pFormat, pArgumentsCopy);
+   va_end (pArgumentsCopy);
 }
 
+// ------------------------------------
+//        Translator extension
+// ------------------------------------
+
+class t_MainTranslator : public QTranslator    // Has been extended initially to support configuration file driven
+{                                              // translation of text UserField, might be used for others as well.
+   public:
+      t_MainTranslator (QObject *pParent=0) : QTranslator (pParent) {} ;
+
+      void Insert (const QString &Source, const QString &Translation)
+      {
+         (void) oDict.insert (Source, Translation);
+      }
+
+      QString translate (const char *pContext, const char *pSourceText, const char *pDisambiguation = 0) const
+      {
+         QString Source(pSourceText);
+
+         if (oDict.contains (Source))
+              return oDict.value (Source);
+         else return QTranslator::translate (pContext, pSourceText, pDisambiguation);
+      }
+
+      bool isEmpty (void) const
+      {
+         return oDict.isEmpty();
+      }
+
+   private :
+      QHash<QString, QString> oDict;
+};
+
 // ------------------------------------
 //           Main program
 // ------------------------------------
@@ -223,6 +259,17 @@ APIRET MainPocessEvents (void)
    return NO_ERROR;
 }
 
+static void ShowSplashText (QSplashScreen *pSplashScreen, QString Text)
+{
+   if (pSplashScreen)
+   {
+      pSplashScreen->showMessage (Text, Qt::AlignHCenter | Qt::AlignBottom, Qt::white);
+      QApplication::processEvents ();
+      CHK_EXIT (QtUtilSleep (MAIN_SHOW_SPLASH_TIME_EVENT))
+      QApplication::processEvents ();
+   }
+}
+
 static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
 {
    QSplashScreen *pSplashScreen = NULL;
@@ -234,7 +281,6 @@ static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
    APIRET          Err = NO_ERROR;
    APIRET          rc;
    int             RetCode;
-//   const char    *pLibEwfVersionCompiled;
    const char    *pLibEwfVersionInstalled;
    const char    *pLibGuyToolsVersionInstalled;
    quint64         FifoAllocs;
@@ -255,6 +301,7 @@ static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
    CHK_EXIT (ErrorInit())
    CHK_EXIT (TOOL_ERROR_REGISTER_CODE   (ERROR_MAIN_UNKNOWN_STARTUP_SIZE   ))
    CHK_EXIT (TOOL_ERROR_REGISTER_CODE   (ERROR_MAIN_INVALID_WINDOW_SIZE    ))
+   CHK_EXIT (TOOL_ERROR_REGISTER_CODE   (ERROR_MAIN_INVALID_NUMBERSTYLE    ))
    CHK_EXIT (TOOL_ERROR_REGISTER_CODE   (ERROR_MAIN_LIBEWF_VERSION_MISMATCH))
 
    CHK_EXIT (ToolErrorSetLogFn          (&t_Log::vEntryError))
@@ -275,18 +322,8 @@ static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
    LOG_INFO ("Libguytools version installed on this PC: %s", pLibGuyToolsVersionInstalled)
    LOG_INFO ("Libewf      version installed on this PC: %s", pLibEwfVersionInstalled)
 
-
    unsigned long long Bytes = UtilGetInstalledRAM ();
    LOG_INFO ("Total amount of memory installed: %0.1f MB", Bytes/(1024.0*1024.0))
-   
-//   pLibEwfVersionCompiled = LIBEWF_VERSION_STRING;
-//
-//   LOG_INFO ("Compiled for libewf version:         %s", pLibEwfVersionCompiled)
-//   if (strcmp (pLibEwfVersionInstalled, pLibEwfVersionCompiled))
-//   {
-//      LOG_ERROR ("Libewf version mismatch")
-//      CHK_CONST (ERROR_MAIN_LIBEWF_VERSION_MISMATCH)
-//   }
 
    // Initialise QApplication
    // -----------------------
@@ -298,7 +335,6 @@ static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
 
    // Check Cfg file
    // --------------
-//   CHK (ToolCfgUseAdjustedCommandLine (MainLocal.pApp->argc(), MainLocal.pApp->argv()))
    CHK (ToolCfgUseAdjustedCommandLine (QApplication::argc(), QApplication::argv()))
    CHK (ToolCfgSetLogFn (&MainCfgLog))
    CHK (CfgInit())
@@ -332,21 +368,22 @@ static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
    {
       pSplashScreen = new QSplashScreen (*pSplashPixmap);
       pSplashScreen->show();
-      QApplication::processEvents ();
-      CHK_EXIT (QtUtilSleep (MAIN_SHOW_SPLASH_TIME))
+      ShowSplashText (pSplashScreen, "Reading configuration");
    }
    CHK (CfgReadConfiguration (pCfgFileName))
 
    // Language selection
    // ------------------
-   QTranslator TranslatorQt;
-   QTranslator TranslatorGuymager;
-   QString     LangNameQt= QString(MAIN_LANGUAGE_FILENAME_QT) + CONFIG(Language);
-   QString     LangName  = QString(MAIN_LANGUAGE_FILENAME   ) + CONFIG(Language);
-   QString     LangDirQt = MAIN_DEFAULT_LANGUAGE_DIR_QT;
-   QString     LangDir   = MAIN_DEFAULT_MISCFILES_DIR;
-   QString     TryName, TryDir;
-   bool        Found;
+   ShowSplashText (pSplashScreen, "Setting up language");
+   QTranslator      TranslatorQt;
+   QTranslator      TranslatorGuymager;
+   t_MainTranslator TranslatorDynamic;
+   QString          LangNameQt= QString(MAIN_LANGUAGE_FILENAME_QT) + CONFIG(Language);
+   QString          LangName  = QString(MAIN_LANGUAGE_FILENAME   ) + CONFIG(Language);
+   QString          LangDirQt = MAIN_DEFAULT_LANGUAGE_DIR_QT;
+   QString          LangDir   = MAIN_DEFAULT_MISCFILES_DIR;
+   QString          TryName, TryDir;
+   bool             Found;
 
    #ifdef LANGUAGE_DIR_QT
       LangDirQt = LANGUAGE_DIR_QT;
@@ -405,6 +442,9 @@ static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
    {
       LOG_INFO ("Language file %s not found in %s. Not using any language file.", QSTR_TO_PSZ(TryName), QSTR_TO_PSZ(TryDir))
    }
+   if (strlen (CONFIG(UserFieldName)))
+      TranslatorDynamic.Insert (MAINWINDOW_COLUMN_UserField, CONFIG(UserFieldName));
+   MainLocal.pApp->installTranslator (&TranslatorDynamic);
 
    // Check real and effective user ID
    // --------------------------------
@@ -419,19 +459,26 @@ static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
    }
    else
    {
-      LOG_INFO ("Running with non-root rights, asking user...")
-      Button = QMessageBox::question (NULL, QObject::tr ("Missing root rights", "Dialog title"),
-                                            QObject::tr ("Guymager needs to be started with root rights in order to perform acquisitions. "
-                                                         "You are not root and thus won't be able to acquire devices.\nContinue anyway?"),
-                                                          QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
-      if (Button == QMessageBox::Yes)
+      if (CONFIG(CheckRootRights))
       {
-         LOG_INFO ("User continues anyway")
+         LOG_INFO ("Running with non-root rights, asking user...")
+         Button = t_MessageBox::question (NULL, QObject::tr ("Missing root rights", "Dialog title"),
+                                                QObject::tr ("Guymager needs to be started with root rights in order to perform acquisitions. "
+                                                             "You are not root and thus won't be able to acquire devices.\nContinue anyway?"),
+                                                              QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
+         if (Button == QMessageBox::Yes)
+         {
+            LOG_INFO ("User continues anyway")
+         }
+         else
+         {
+            LOG_INFO ("User doesn't want to continue. Exiting now.")
+            return NO_ERROR;
+         }
       }
       else
       {
-         LOG_INFO ("User doesn't want to continue. Exiting now.")
-         return NO_ERROR;
+         LOG_INFO ("Running with non-root rights, continuing without asking user (CheckRootRights is off)")
       }
    }
 
@@ -456,15 +503,25 @@ static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
       case CFG_NUMBERSTYLE_DECIMAL_COMMA: MainLocal.pNumberLocale = new QLocale("de_DE"); break;
       case CFG_NUMBERSTYLE_DECIMAL_POINT: MainLocal.pNumberLocale = new QLocale("C"    ); break;
       case CFG_NUMBERSTYLE_LOCALE       : MainLocal.pNumberLocale = new QLocale(       ); break;
-      default: CHK_CONST (ERROR_MAIN_INVALID_WINDOW_SIZE)
+      default: CHK_CONST (ERROR_MAIN_INVALID_NUMBERSTYLE)
    }
 
    // Initialise libewf
    // -----------------
-   libewf_set_notify_values (stderr, CONFIG (VerboseLibewf) ? 1 : 0);
+   #if ((LIBEWF_VERSION != 20100226) && (LIBEWF_VERSION != 20080501) && (LIBEWF_VERSION != 20111015) && (LIBEWF_VERSION != 20130416))
+      #error "Please check EWF documentation for newer Encase formats and adjust following code"
+   #endif
+
+   #if (LIBEWF_VERSION >= 20130416)
+      libewf_notify_set_stream  (stderr, NULL);
+      libewf_notify_set_verbose (CONFIG (VerboseLibewf) ? 1 : 0);
+   #else
+      libewf_set_notify_values (stderr, CONFIG (VerboseLibewf) ? 1 : 0);
+   #endif
 
    // Create central data structures
    // ------------------------------
+   ShowSplashText (pSplashScreen, "Creating data structures");
    MainLocal.pDeviceList = new t_DeviceList;
 
    // Create main window, remove splashscreen and enter main loop
