@@ -1,7 +1,7 @@
 #!/bin/bash
 
 TH_Date=`date '+%Y-%m-%d'`
-TH_Source=`head -n 1 ../debian/changelog | awk '{
+TH_Source=`head -qn 1 ../debian/changelog ../changelog 2>/dev/null | awk '{
                                                   Version = $2
                                                   gsub ("\\\\(", "", Version)
                                                   gsub ("\\\\)", "", Version)
