@@ -9,7 +9,7 @@
 //  Module:         MD5 calculation
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -51,8 +51,13 @@ typedef struct
 
 // Always make sure that these functions are compiled with O3 optimisation or
 // else, performance is about 5 times worse! See also MD5ProcessBlock in md5.cpp.
-void MD5Init   (t_pMD5Context pContext)                                   __attribute__((optimize("-O3")));
-void MD5Append (t_pMD5Context pContext, const void *pBuffer, size_t Len)  __attribute__((optimize("-O3")));
-void MD5Finish (t_pMD5Context pContext, void *pDigest)                    __attribute__((optimize("-O3")));
+
+//void MD5Init   (t_pMD5Context pContext)                                   __attribute__((optimize("-O3")));
+//void MD5Append (t_pMD5Context pContext, const void *pBuffer, size_t Len)  __attribute__((optimize("-O3")));
+//void MD5Finish (t_pMD5Context pContext, void *pDigest)                    __attribute__((optimize("-O3")));
+
+void MD5Init   (t_pMD5Context pContext);
+void MD5Append (t_pMD5Context pContext, const void *pBuffer, size_t Len);
+void MD5Finish (t_pMD5Context pContext, void *pDigest);
 
 #endif
