@@ -9,7 +9,7 @@
 //  Module:         Different Qt utility functions
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -254,6 +254,7 @@ APIRET QtUtilSleep (int MilliSeconds)
 //         Default widget setting
 // ------------------------------------
 
+#ifdef OLDOLD
 APIRET QtUtilAdjustPushButton (QPushButton *pWidget)
 {
    pWidget->setFont (CONFIG_FONT (FONTOBJECT_DIALOGDEFAULT));
@@ -305,12 +306,12 @@ APIRET QtUtilAdjustDefaults (QLayout *pLayout, bool TopLevelLayout)
 
    return NO_ERROR;
 }
-
+#endif
 // ------------------------------------
 //          External calls
 // ------------------------------------
 
-APIRET QtUtilProcessCommand (const QString &Command, QString &StandardOutput)
+APIRET QtUtilProcessCommand (const QString &Command, QString *pStandardOutput)
 {
    QProcess    Process;
    QStringList SplittedCommand;
@@ -358,8 +359,9 @@ APIRET QtUtilProcessCommand (const QString &Command, QString &StandardOutput)
    {
 //    StandardOutput  = Process.readAllStandardOutput();
 //    StandardOutput  = Process.readAll();
-      StandardOutput  = QString::fromUtf8 (Process.readAll().constData());
-      
+      if (pStandardOutput)
+         *pStandardOutput = QString::fromUtf8 (Process.readAll().constData());
+
       StrState = "Done";
    }
    else
