@@ -9,7 +9,7 @@
 //  Module:         Thread for calculating hashes
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -76,7 +76,9 @@ void t_ThreadHash::run (void)
    quint64           *pBlocks;
    quint64             BlocksCalculated = 0;
    quint64             BlocksVerified   = 0;
+   quint64             Hashed           = 0;
    t_HashContextMD5    HashContextMD5;
+   t_HashContextSHA1   HashContextSHA1;
    t_HashContextSHA256 HashContextSHA256;
    bool                VerifyLoop = false;
 
@@ -88,8 +90,9 @@ void t_ThreadHash::run (void)
    {
       *pBlocks = 0;
       Finished = false;
-      if (pDevice->Acquisition.CalcMD5   ) CHK_EXIT (HashMD5Init    (&HashContextMD5   ))
-      if (pDevice->Acquisition.CalcSHA256) CHK_EXIT (HashSHA256Init (&HashContextSHA256))
+      if (pDevice->Acquisition1.CalcMD5   ) CHK_EXIT (HashMD5Init    (&HashContextMD5   ))
+      if (pDevice->Acquisition1.CalcSHA1  ) CHK_EXIT (HashSHA1Init   (&HashContextSHA1  ))
+      if (pDevice->Acquisition1.CalcSHA256) CHK_EXIT (HashSHA256Init (&HashContextSHA256))
       do
       {
          CHK_EXIT (SetDebugMessage ("Wait for block"))
@@ -98,15 +101,23 @@ void t_ThreadHash::run (void)
          if (pFifoBlock)
          {
             (*pBlocks)++;
-            if (pDevice->Acquisition.CalcMD5   ) CHK_EXIT (HashMD5Append    (&HashContextMD5   , pFifoBlock->Buffer, pFifoBlock->DataSize))
-            if (pDevice->Acquisition.CalcSHA256) CHK_EXIT (HashSHA256Append (&HashContextSHA256, pFifoBlock->Buffer, pFifoBlock->DataSize))
+            if (pDevice->Acquisition1.CalcMD5   ) CHK_EXIT (HashMD5Append    (&HashContextMD5   , pFifoBlock->Buffer, pFifoBlock->DataSize))
+            if (pDevice->Acquisition1.CalcSHA1  ) CHK_EXIT (HashSHA1Append   (&HashContextSHA1  , pFifoBlock->Buffer, pFifoBlock->DataSize))
+            if (pDevice->Acquisition1.CalcSHA256) CHK_EXIT (HashSHA256Append (&HashContextSHA256, pFifoBlock->Buffer, pFifoBlock->DataSize))
             if (VerifyLoop)
             {
-               pDevice->IncCurrentVerifyPosSrc (pFifoBlock->DataSize);
+               pDevice->CurrentVerifyPosSrc.Inc (pFifoBlock->DataSize);
                CHK_EXIT (t_Fifo::Destroy (pDevice->pFifoMemory, pFifoBlock))
             }
             else
             {
+               Hashed += pFifoBlock->DataSize;
+               if (Hashed >= pDevice->Size)  // Make sure to calculate the hash before inserting the last block or else, if the write thread
+               {                             // get a time slice in between, it could possibly put an invalid hash into the image file.
+                  if (pDevice->Acquisition1.CalcMD5   ) CHK_EXIT (HashMD5Digest    (&HashContextMD5   , &pDevice->MD5Digest   ))
+                  if (pDevice->Acquisition1.CalcSHA1  ) CHK_EXIT (HashSHA1Digest   (&HashContextSHA1  , &pDevice->SHA1Digest  ))
+                  if (pDevice->Acquisition1.CalcSHA256) CHK_EXIT (HashSHA256Digest (&HashContextSHA256, &pDevice->SHA256Digest))
+               }
                CHK_EXIT (pDevice->pFifoHashOut->Insert (pFifoBlock))
             }
          }
@@ -115,41 +126,36 @@ void t_ThreadHash::run (void)
             LOG_INFO ("[%s] Dummy block", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
             Finished = true;
          }
-      } while (!Finished && !pDevice->AbortRequest);
-      if (pDevice->AbortRequest)
+      } while (!Finished && !pDevice->Error.Abort());
+      if (pDevice->Error.Abort())
       {
          break;
       }
       else if (VerifyLoop)
       {
-         if (pDevice->Acquisition.CalcMD5   ) CHK_EXIT (HashMD5Digest    (&HashContextMD5   , &pDevice->MD5DigestVerifySrc   ))
-         if (pDevice->Acquisition.CalcSHA256) CHK_EXIT (HashSHA256Digest (&HashContextSHA256, &pDevice->SHA256DigestVerifySrc))
+         if (pDevice->Acquisition1.CalcMD5   ) CHK_EXIT (HashMD5Digest    (&HashContextMD5   , &pDevice->MD5DigestVerifySrc   ))
+         if (pDevice->Acquisition1.CalcSHA1  ) CHK_EXIT (HashSHA1Digest   (&HashContextSHA1  , &pDevice->SHA1DigestVerifySrc  ))
+         if (pDevice->Acquisition1.CalcSHA256) CHK_EXIT (HashSHA256Digest (&HashContextSHA256, &pDevice->SHA256DigestVerifySrc))
          break;
       }
+      else if (pDevice->Acquisition1.VerifySrc)
+      {
+         pBlocks = &BlocksVerified;
+         VerifyLoop = true;
+         CHK_EXIT (pDevice->pFifoHashOut->InsertDummy ())  // Make compression threads know they can end
+      }
       else
       {
-         if (pDevice->Acquisition.CalcMD5   ) CHK_EXIT (HashMD5Digest    (&HashContextMD5   , &pDevice->MD5Digest   ))
-         if (pDevice->Acquisition.CalcSHA256) CHK_EXIT (HashSHA256Digest (&HashContextSHA256, &pDevice->SHA256Digest))
-         if (pDevice->Acquisition.VerifySrc)
-         {
-            pBlocks = &BlocksVerified;
-            VerifyLoop = true;
-            CHK_EXIT (pDevice->pFifoHashOut->InsertDummy ())  // Make compression threads know they can end
-         }
-         else
-         {
-            break;
-         }
+         break;
       }
    }
 
-   if (pDevice->Acquisition.VerifySrc)
+   if (pDevice->Acquisition1.VerifySrc)
         LOG_INFO ("[%s] Hash thread exits now - %Ld blocks processed, %Ld blocks verified", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), BlocksCalculated, BlocksVerified)
    else LOG_INFO ("[%s] Hash thread exits now - %Ld blocks processed"                     , QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), BlocksCalculated)
    CHK_EXIT (SetDebugMessage ("Exit run function"))
 }
 
-
 void t_ThreadHash::SlotFinished (void)
 {
    emit SignalEnded (pOwn->pDevice);
