@@ -9,7 +9,7 @@
 //  Module:         Different utility functions
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -42,9 +42,21 @@ unsigned int UtilGetMaxZcompressedBufferSize (unsigned int UncompressedBufferSiz
 unsigned long long UtilGetInstalledRAM (void); // Number of bytes
 unsigned int       UtilGetNumberOfCPUs (void);
 
+APIRET UtilGetFreeSpace (const QString &DirPath, quint64 *pFreeBytes);
+
+QString UtilSizeToHuman (unsigned long long Size, bool SI=true, int FracLen=-1, int UnitThreshold=-1);
+
+
 APIRET UtilInit   (void);
 APIRET UtilDeInit (void);
 
+// ------------------------------------
+//             Error codes
+// ------------------------------------
+
+enum
+{
+   ERROR_UTIL_FREESPACE_UNKNOWN = ERROR_BASE_UTIL + 1
+};
+
 #endif
-
-
